# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


class Mish(Base):
    @staticmethod
    def export() -> None:
        node = onnx.helper.make_node("Mish", inputs=["X"], outputs=["Y"])

        input_data = np.linspace(-10, 10, 10000, dtype=np.float32)

        # Calculate expected output data
        expected_output = input_data * np.tanh(np.log1p(np.exp(input_data)))

        expect(node, inputs=[input_data], outputs=[expected_output], name="test_mish")



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

from onnx.reference.ops._op_common_random import _CommonRandom


class RandomUniform(_CommonRandom):
    def _run(self, dtype=None, high=None, low=None, seed=None, shape=None):
        dtype = self._dtype(dtype=dtype)
        state = self._get_state(seed)
        res = state.rand(*shape).astype(dtype)
        res *= high - low
        res += low
        return (res.astype(dtype),)



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect
from onnx.backend.test.case.utils import all_numeric_dtypes


class Max(Base):
    @staticmethod
    def export() -> None:
        data_0 = np.array([3, 2, 1]).astype(np.float32)
        data_1 = np.array([1, 4, 4]).astype(np.float32)
        data_2 = np.array([2, 5, 3]).astype(np.float32)
        result = np.array([3, 5, 4]).astype(np.float32)
        node = onnx.helper.make_node(
            "Max",
            inputs=["data_0", "data_1", "data_2"],
            outputs=["result"],
        )
        expect(
            node,
            inputs=[data_0, data_1, data_2],
            outputs=[result],
            name="test_max_example",
        )

        node = onnx.helper.make_node(
            "Max",
            inputs=["data_0"],
            outputs=["result"],
        )
        expect(node, inputs=[data_0], outputs=[data_0], name="test_max_one_input")

        result = np.maximum(data_0, data_1)
        node = onnx.helper.make_node(
            "Max",
            inputs=["data_0", "data_1"],
            outputs=["result"],
        )
        expect(
            node, inputs=[data_0, data_1], outputs=[result], name="test_max_two_inputs"
        )

    @staticmethod
    def export_max_all_numeric_types() -> None:
        for op_dtype in all_numeric_dtypes:
            data_0 = np.array([3, 2, 1]).astype(op_dtype)
            data_1 = np.array([1, 4, 4]).astype(op_dtype)
            result = np.array([3, 4, 4]).astype(op_dtype)
            node = onnx.helper.make_node(
                "Max",
                inputs=["data_0", "data_1"],
                outputs=["result"],
            )
            expect(
                node,
                inputs=[data_0, data_1],
                outputs=[result],
                name=f"test_max_{np.dtype(op_dtype).name}",
            )



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.op_run import OpRun

_acceptable_str_dtypes = ("U", "O")


class StringConcat(OpRun):
    def _run(self, x, y):
        if (
            x.dtype.kind not in _acceptable_str_dtypes
            or y.dtype.kind not in _acceptable_str_dtypes
        ):
            raise TypeError(
                f"Inputs must be string tensors, received dtype {x.dtype} and {y.dtype}"
            )
        # As per onnx/mapping.py, object numpy dtype corresponds to TensorProto.STRING
        return (np.char.add(x.astype(np.str_), y.astype(np.str_)).astype(object),)



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
# type: ignore
from __future__ import annotations

import importlib
import itertools
import os
import unittest

import numpy as np
from numpy.testing import assert_allclose
from parameterized import parameterized

import onnx
from onnx import ONNX_ML, TensorProto, TypeProto, ValueInfoProto
from onnx.helper import (
    make_graph,
    make_model_gen_version,
    make_node,
    make_opsetid,
    make_tensor,
    make_tensor_value_info,
)
from onnx.reference import ReferenceEvaluator
from onnx.reference.ops.aionnxml.op_tree_ensemble import (
    AggregationFunction,
    Mode,
    PostTransform,
)

# TODO (https://github.com/microsoft/onnxruntime/issues/14932): Get max supported version from onnxruntime directly
# For now, bump the version in CIs whenever there is a new onnxruntime release
ORT_MAX_IR_SUPPORTED_VERSION = int(os.getenv("ORT_MAX_IR_SUPPORTED_VERSION", "8"))
ORT_MAX_ONNX_OPSET_SUPPORTED_VERSION = int(
    os.getenv("ORT_MAX_ONNX_OPSET_SUPPORTED_VERSION", "18")
)
ORT_MAX_ML_OPSET_SUPPORTED_VERSION = int(
    os.getenv("ORT_MAX_ML_OPSET_SUPPORTED_VERSION", "3")
)

TARGET_OPSET = onnx.defs.onnx_opset_version() - 2
TARGET_OPSET_ML = onnx.defs.onnx_ml_opset_version()
OPSETS = [make_opsetid("", TARGET_OPSET), make_opsetid("ai.onnx.ml", TARGET_OPSET_ML)]


def has_onnxruntime():
    return importlib.util.find_spec("onnxruntime") is not None


class TestReferenceEvaluatorAiOnnxMl(unittest.TestCase):
    @staticmethod
    def _check_ort(model, feeds, atol=0, rtol=0, equal=False, rev=False):
        if not has_onnxruntime():
            raise unittest.SkipTest("onnxruntime not installed")

        from onnxruntime import InferenceSession  # noqa: PLC0415

        onnx_domain_opset = ORT_MAX_ONNX_OPSET_SUPPORTED_VERSION
        ml_domain_opset = ORT_MAX_ML_OPSET_SUPPORTED_VERSION
        for opset in model.opset_import:
            if opset.domain in ("", "ai.onnx"):
                onnx_domain_opset = opset.version
                break
        for opset in model.opset_import:
            if opset.domain == "ai.onnx.ml":
                ml_domain_opset = opset.version
                break
        # The new IR or opset version is not supported by onnxruntime yet
        if (
            model.ir_version > ORT_MAX_IR_SUPPORTED_VERSION
            or onnx_domain_opset > ORT_MAX_ONNX_OPSET_SUPPORTED_VERSION
            or ml_domain_opset > ORT_MAX_ML_OPSET_SUPPORTED_VERSION
        ):
            return

        ort = InferenceSession(
            model.SerializeToString(), providers=["CPUExecutionProvider"]
        )
        sess = ReferenceEvaluator(model)
        expected = ort.run(None, feeds)
        got = sess.run(None, feeds)
        if len(expected) != len(got):
            raise AssertionError(
                f"onnxruntime returns a different number of output "
                f"{len(expected)} != {len(sess)} (ReferenceEvaluator)."
            )
        look = (
            zip(reversed(expected), reversed(got), strict=True)
            if rev
            else zip(expected, got, strict=True)
        )
        for i, (e, g) in enumerate(look):
            if e.shape != g.shape:
                raise AssertionError(
                    f"Unexpected shape {g.shape} for output {i} "
                    f"(expecting {e.shape})\n{e!r}\n---\n{g!r}."
                )
            if equal:
                if e.tolist() != g.tolist():
                    raise AssertionError(
                        f"Discrepancies for output {i}"
                        f"\nexpected=\n{e}\n!=\nresults=\n{g}"
                    )
            else:
                assert_allclose(
                    actual=g,
                    desired=e,
                    atol=atol,
                    rtol=rtol,
                    err_msg=f"Discrepancies for output {i} expected[0]={e.ravel()[0]}.",
                )

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_binarizer(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node("Binarizer", ["X"], ["Y"], threshold=5.5, domain="ai.onnx.ml")
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.arange(12).reshape((3, 4)).astype(np.float32)
        expected = np.array(
            [[0, 0, 0, 0], [0, 0, 1, 1], [1, 1, 1, 1]], dtype=np.float32
        )
        self._check_ort(model, {"X": x})
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        assert_allclose(got, expected)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_scaler(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "Scaler", ["X"], ["Y"], scale=[0.5], offset=[-4.5], domain="ai.onnx.ml"
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.arange(12).reshape((3, 4)).astype(np.float32)
        expected = np.array(
            [
                [2.25, 2.75, 3.25, 3.75],
                [4.25, 4.75, 5.25, 5.75],
                [6.25, 6.75, 7.25, 7.75],
            ],
            dtype=np.float32,
        )
        self._check_ort(model, {"X": x})
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        assert_allclose(got, expected)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_array_feature_extractor(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        A = make_tensor_value_info("A", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "ArrayFeatureExtractor", ["X", "A"], ["Y"], domain="ai.onnx.ml"
        )
        graph = make_graph([node1], "ml", [X, A], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.arange(12).reshape((3, 4)).astype(np.float32)

        expected = np.array([[0, 4, 8]], dtype=np.float32).T
        feeds = {"X": x, "A": np.array([0], dtype=np.int64)}
        self._check_ort(model, feeds)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, feeds)[0]
        assert_allclose(got, expected)

        expected = np.array([[0, 4, 8], [1, 5, 9]], dtype=np.float32).T
        feeds = {"X": x, "A": np.array([0, 1], dtype=np.int64)}
        self._check_ort(model, feeds)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, feeds)[0]
        assert_allclose(got, expected)

        expected = np.array(
            [[0, 4, 8], [1, 5, 9], [0, 4, 8], [1, 5, 9], [0, 4, 8], [1, 5, 9]],
            dtype=np.float32,
        ).T
        feeds = {"X": x, "A": np.array([0, 1, 0, 1, 0, 1], dtype=np.int64)}
        self._check_ort(model, feeds)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, feeds)[0]
        assert_allclose(got, expected)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_normalizer(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        x = np.arange(12).reshape((3, 4)).astype(np.float32)
        expected = {
            "MAX": x / x.max(axis=1, keepdims=1),
            "L1": x / np.abs(x).sum(axis=1, keepdims=1),
            "L2": x / (x**2).sum(axis=1, keepdims=1) ** 0.5,
        }
        for norm, value in expected.items():
            with self.subTest(norm=norm):
                node1 = make_node(
                    "Normalizer", ["X"], ["Y"], norm=norm, domain="ai.onnx.ml"
                )
                graph = make_graph([node1], "ml", [X], [Y])
                model = make_model_gen_version(graph, opset_imports=OPSETS)
                onnx.checker.check_model(model)

                feeds = {"X": x}
                self._check_ort(model, feeds, atol=1e-6)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, feeds)[0]
                assert_allclose(got, value, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_feature_vectorizer(self):
        X = [
            make_tensor_value_info("X0", TensorProto.FLOAT, [None, None]),
            make_tensor_value_info("X1", TensorProto.FLOAT, [None, None]),
        ]
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        x = [
            np.arange(9).reshape((3, 3)).astype(np.float32),
            np.arange(9).reshape((3, 3)).astype(np.float32) + 0.5,
        ]
        expected = {
            (1,): np.array([[0], [3], [6]], dtype=np.float32),
            (2,): np.array([[0, 1], [3, 4], [6, 7]], dtype=np.float32),
            (4,): np.array(
                [[0, 1, 2, 0], [3, 4, 5, 0], [6, 7, 8, 0]], dtype=np.float32
            ),
            (1, 1): np.array([[0, 0.5], [3, 3.5], [6, 6.5]], dtype=np.float32),
            (0, 1): np.array([[0.5], [3.5], [6.5]], dtype=np.float32),
        }
        for inputdimensions, value in expected.items():
            att = (
                list(inputdimensions)
                if isinstance(inputdimensions, tuple)
                else inputdimensions
            )
            with self.subTest(inputdimensions=att):
                node1 = make_node(
                    "FeatureVectorizer",
                    [f"X{i}" for i in range(len(att))],
                    ["Y"],
                    inputdimensions=att,
                    domain="ai.onnx.ml",
                )
                graph = make_graph([node1], "ml", X[: len(att)], [Y])
                model = make_model_gen_version(graph, opset_imports=OPSETS)
                onnx.checker.check_model(model)

                feeds = {f"X{i}": v for i, v in enumerate(x[: len(att)])}
                self._check_ort(model, feeds, atol=1e-6)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, feeds)[0]
                assert_allclose(got, value, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_imputer_float(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "Imputer",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            imputed_value_floats=np.array([0], dtype=np.float32),
            replaced_value_float=np.nan,
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[0, 1, np.nan, 3]], dtype=np.float32).T
        expected = np.array([[0, 1, 0, 3]], dtype=np.float32).T
        self._check_ort(model, {"X": x})
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        assert_allclose(got, expected)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_imputer_float_2d(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "Imputer",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            imputed_value_floats=np.array([0, 0.1], dtype=np.float32),
            replaced_value_float=np.nan,
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[0, 1, np.nan, 3], [0, 1, np.nan, 3]], dtype=np.float32).T
        expected = np.array([[0, 1, 0, 3], [0, 1, 0.1, 3]], dtype=np.float32).T
        self._check_ort(model, {"X": x})
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        assert_allclose(got, expected)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_imputer_int(self):
        X = make_tensor_value_info("X", TensorProto.INT64, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.INT64, [None, None])
        node1 = make_node(
            "Imputer",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            imputed_value_int64s=np.array([0], dtype=np.int64),
            replaced_value_int64=-1,
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[0, 1, -1, 3]], dtype=np.int64).T
        expected = np.array([[0, 1, 0, 3]], dtype=np.int64).T
        self._check_ort(model, {"X": x})
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        assert_allclose(got, expected)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_label_encoder_float_int(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.INT64, [None, None])
        node1 = make_node(
            "LabelEncoder",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            default_int64=-5,
            keys_floats=[4.0, 1.0, 2.0, 3.0],
            values_int64s=[0, 1, 2, 3],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[0, 1, np.nan, 3, 4]], dtype=np.float32).T
        expected = np.array([[-5, 1, -5, 3, 0]], dtype=np.int64).T
        self._check_ort(model, {"X": x})
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        assert_allclose(got, expected)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_label_encoder_int_string(self):
        X = make_tensor_value_info("X", TensorProto.INT64, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.STRING, [None, None])
        node1 = make_node(
            "LabelEncoder",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            default_string="NONE",
            keys_int64s=[1, 2, 3, 4],
            values_strings=["a", "b", "cc", "ddd"],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[0, 1, 3, 4]], dtype=np.int64).T
        expected = np.array([["NONE"], ["a"], ["cc"], ["ddd"]])
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        self.assertEqual(expected.tolist(), got.tolist())

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_label_encoder_int_string_tensor_attributes(self):
        X = make_tensor_value_info("X", TensorProto.INT64, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.STRING, [None, None])
        node = make_node(
            "LabelEncoder",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            keys_tensor=make_tensor(
                "keys_tensor", TensorProto.INT64, [4], [1, 2, 3, 4]
            ),
            values_tensor=make_tensor(
                "values_tensor", TensorProto.STRING, [4], ["a", "b", "cc", "ddd"]
            ),
            default_tensor=make_tensor(
                "default_tensor", TensorProto.STRING, [], ["NONE"]
            ),
        )
        graph = make_graph([node], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[0, 1, 3, 4]], dtype=np.int64).T
        expected = np.array([["NONE"], ["a"], ["cc"], ["ddd"]])
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        self.assertEqual(expected.tolist(), got.tolist())

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_dict_vectorizer(self):
        value_type = TypeProto()
        value_type.tensor_type.elem_type = TensorProto.INT64
        onnx_type = TypeProto()
        onnx_type.map_type.key_type = TensorProto.STRING
        onnx_type.map_type.value_type.CopyFrom(value_type)
        value_info = ValueInfoProto()
        value_info.name = "X"
        value_info.type.CopyFrom(onnx_type)

        X = value_info
        Y = make_tensor_value_info("Y", TensorProto.INT64, [None, None])
        node1 = make_node(
            "DictVectorizer",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            string_vocabulary=["a", "c", "b", "z"],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = {"a": np.array(4, dtype=np.int64), "c": np.array(8, dtype=np.int64)}
        expected = np.array([4, 8, 0, 0], dtype=np.int64)
        # Unexpected input data type. Actual: ((map(string,tensor(float)))) , expected: ((map(string,tensor(int64))))
        # self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        self.assertEqual(expected.tolist(), got.tolist())

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_one_hot_encoder_int(self):
        X = make_tensor_value_info("X", TensorProto.INT64, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None, None])
        node1 = make_node(
            "OneHotEncoder",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            zeros=1,
            cats_int64s=[1, 2, 3],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[5, 1, 3], [2, 1, 3]], dtype=np.int64)
        expected = np.array(
            [[[0, 0, 0], [1, 0, 0], [0, 0, 1]], [[0, 1, 0], [1, 0, 0], [0, 0, 1]]],
            dtype=np.float32,
        )
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        self.assertEqual(expected.tolist(), got.tolist())

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_one_hot_encoder_string(self):
        X = make_tensor_value_info("X", TensorProto.STRING, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None, None])
        node1 = make_node(
            "OneHotEncoder",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            zeros=1,
            cats_strings=["c1", "c2", "c3"],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([["c5", "c1", "c3"], ["c2", "c1", "c3"]])
        expected = np.array(
            [[[0, 0, 0], [1, 0, 0], [0, 0, 1]], [[0, 1, 0], [1, 0, 0], [0, 0, 1]]],
            dtype=np.float32,
        )
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        self.assertEqual(expected.tolist(), got.tolist())

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_one_hot_encoder_zeros(self):
        X = make_tensor_value_info("X", TensorProto.INT64, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None, None])
        node1 = make_node(
            "OneHotEncoder",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            zeros=0,
            cats_int64s=[1, 2, 3],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.array([[2, 1, 3], [2, 1, 3]], dtype=np.int64)
        expected = np.array(
            [[[0, 1, 0], [1, 0, 0], [0, 0, 1]], [[0, 1, 0], [1, 0, 0], [0, 0, 1]]],
            dtype=np.float32,
        )
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})[0]
        self.assertEqual(expected.tolist(), got.tolist())

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_linear_regressor(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "LinearRegressor",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            coefficients=[0.3, -0.77],
            intercepts=[0.5],
            post_transform="NONE",
            targets=1,
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.arange(6).reshape((-1, 2)).astype(np.float32)
        expected = np.array([[-0.27], [-1.21], [-2.15]], dtype=np.float32)
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})
        assert_allclose(got[0], expected, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_linear_regressor_2(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "LinearRegressor",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            coefficients=[0.3, -0.77, 0.3, -0.77],
            intercepts=[0.5, 0.7],
            post_transform="NONE",
            targets=2,
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        x = np.arange(6).reshape((-1, 2)).astype(np.float32)
        expected = np.array(
            [[-0.27, -0.07], [-1.21, -1.01], [-2.15, -1.95]], dtype=np.float32
        )
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})
        assert_allclose(got[0], expected, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_linear_classifier_multi(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        In = make_tensor_value_info("I", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        expected_post = {
            "NONE": [
                np.array([0, 2, 2], dtype=np.int64),
                np.array(
                    [[2.41, -2.12, 0.59], [0.67, -1.14, 1.35], [-1.07, -0.16, 2.11]],
                    dtype=np.float32,
                ),
            ],
            "LOGISTIC": [
                np.array([0, 2, 2], dtype=np.int64),
                np.array(
                    [
                        [0.917587, 0.107168, 0.643365],
                        [0.661503, 0.24232, 0.79413],
                        [0.255403, 0.460085, 0.891871],
                    ],
                    dtype=np.float32,
                ),
            ],
            "SOFTMAX": [
                np.array([0, 2, 2], dtype=np.int64),
                np.array(
                    [
                        [0.852656, 0.009192, 0.138152],
                        [0.318722, 0.05216, 0.629118],
                        [0.036323, 0.090237, 0.87344],
                    ],
                    dtype=np.float32,
                ),
            ],
            "SOFTMAX_ZERO": [
                np.array([0, 2, 2], dtype=np.int64),
                np.array(
                    [
                        [0.852656, 0.009192, 0.138152],
                        [0.318722, 0.05216, 0.629118],
                        [0.036323, 0.090237, 0.87344],
                    ],
                    dtype=np.float32,
                ),
            ],
            "PROBIT": [
                np.array([1, 1, 1], dtype=np.int64),
                np.array(
                    [
                        [-0.527324, -0.445471, -1.080504],
                        [-0.067731, 0.316014, -0.310748],
                        [0.377252, 1.405167, 0.295001],
                    ],
                    dtype=np.float32,
                ),
            ],
        }
        for post in ("SOFTMAX", "NONE", "LOGISTIC", "SOFTMAX_ZERO", "PROBIT"):
            if post == "PROBIT":
                coefficients = [0.058, 0.029, 0.09, 0.058, 0.029, 0.09]
                intercepts = [0.27, 0.27, 0.05]
            else:
                coefficients = [-0.58, -0.29, -0.09, 0.58, 0.29, 0.09]
                intercepts = [2.7, -2.7, 0.5]
            with self.subTest(post_transform=post):
                node1 = make_node(
                    "LinearClassifier",
                    ["X"],
                    ["I", "Y"],
                    domain="ai.onnx.ml",
                    classlabels_ints=[0, 1, 2],
                    coefficients=coefficients,
                    intercepts=intercepts,
                    multi_class=0,
                    post_transform=post,
                )
                graph = make_graph([node1], "ml", [X], [In, Y])
                model = make_model_gen_version(graph, opset_imports=OPSETS)
                onnx.checker.check_model(model)
                x = np.arange(6).reshape((-1, 2)).astype(np.float32)
                self._check_ort(model, {"X": x}, rev=True, atol=1e-4)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                expected = expected_post[post]
                assert_allclose(got[1], expected[1], atol=1e-4)
                assert_allclose(got[0], expected[0])

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_linear_classifier_binary(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        In = make_tensor_value_info("I", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        expected_post = {
            "NONE": [
                np.array([1, 1], dtype=np.int64),
                np.array([[-9.53, 9.53], [-6.65, 6.65]], dtype=np.float32),
            ],
            "LOGISTIC": [
                np.array([1, 1], dtype=np.int64),
                np.array(
                    [[7.263436e-05, 9.999274e-01], [1.292350e-03, 9.987077e-01]],
                    dtype=np.float32,
                ),
            ],
            "SOFTMAX": [
                np.array([1, 1], dtype=np.int64),
                np.array(
                    [[5.276517e-09, 1.000000e00], [1.674492e-06, 9.999983e-01]],
                    dtype=np.float32,
                ),
            ],
            "SOFTMAX_ZERO": [
                np.array([1, 1], dtype=np.int64),
                np.array(
                    [[5.276517e-09, 1.000000e00], [1.674492e-06, 9.999983e-01]],
                    dtype=np.float32,
                ),
            ],
        }
        x = np.arange(6).reshape((-1, 3)).astype(np.float32)
        for post in ("SOFTMAX", "NONE", "LOGISTIC", "SOFTMAX_ZERO"):
            expected = expected_post[post]
            with self.subTest(post_transform=post):
                node1 = make_node(
                    "LinearClassifier",
                    ["X"],
                    ["I", "Y"],
                    domain="ai.onnx.ml",
                    classlabels_ints=[0, 1],
                    coefficients=[-0.58, -0.29, -0.09],
                    intercepts=[10.0],
                    multi_class=0,
                    post_transform=post,
                )
                graph = make_graph([node1], "ml", [X], [In, Y])
                model = make_model_gen_version(graph, opset_imports=OPSETS)
                onnx.checker.check_model(model)
                # onnxruntime answer seems odd.
                # self._check_ort(model, {"X": x}, rev=True)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_linear_classifier_unary(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        In = make_tensor_value_info("I", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        expected_post = {
            "NONE": [
                np.array([1, 0], dtype=np.int64),
                np.array([[2.23], [-0.65]], dtype=np.float32),
            ],
            "LOGISTIC": [
                np.array([1, 0], dtype=np.int64),
                np.array([[0.902911], [0.34299]], dtype=np.float32),
            ],
            "SOFTMAX": [
                np.array([1, 1], dtype=np.int64),
                np.array([[1.0], [1.0]], dtype=np.float32),
            ],
            "SOFTMAX_ZERO": [
                np.array([1, 1], dtype=np.int64),
                np.array([[1.0], [1.0]], dtype=np.float32),
            ],
        }
        x = np.arange(6).reshape((-1, 3)).astype(np.float32)
        for post in ("NONE", "LOGISTIC", "SOFTMAX_ZERO", "SOFTMAX"):
            expected = expected_post[post]
            with self.subTest(post_transform=post):
                node1 = make_node(
                    "LinearClassifier",
                    ["X"],
                    ["I", "Y"],
                    domain="ai.onnx.ml",
                    classlabels_ints=[1],
                    coefficients=[-0.58, -0.29, -0.09],
                    intercepts=[2.7],
                    multi_class=0,
                    post_transform=post,
                )
                graph = make_graph([node1], "ml", [X], [In, Y])
                model = make_model_gen_version(graph, opset_imports=OPSETS)
                onnx.checker.check_model(model)
                # onnxruntime answer seems odd.
                # self._check_ort(model, {"X": x}, rev=True)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @staticmethod
    def _get_test_tree_ensemble_opset_latest(
        aggregate_function,
        rule=Mode.LEQ,
        unique_targets=False,
        input_type=TensorProto.FLOAT,
    ):
        X = make_tensor_value_info("X", input_type, [None, None])
        Y = make_tensor_value_info("Y", input_type, [None, None])
        if unique_targets:
            weights = [
                1.0,
                10.0,
                100.0,
                1000.0,
                10000.0,
                100000.0,
            ]
        else:
            weights = [
                0.07692307978868484,
                0.5,
                0.5,
                0.0,
                0.2857142984867096,
                0.5,
            ]
        node = make_node(
            "TreeEnsemble",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            n_targets=1,
            aggregate_function=aggregate_function,
            membership_values=None,
            nodes_missing_value_tracks_true=None,
            nodes_hitrates=None,
            post_transform=0,
            tree_roots=[0, 2],
            nodes_splits=make_tensor(
                "node_splits",
                input_type,
                (4,),
                [
                    0.26645058393478394,
                    0.6214364767074585,
                    -0.5592705607414246,
                    -0.7208403944969177,
                ],
            ),
            nodes_featureids=[0, 2, 0, 0],
            nodes_modes=make_tensor(
                "nodes_modes",
                TensorProto.UINT8,
                (4,),
                [rule] * 4,
            ),
            nodes_truenodeids=[1, 0, 3, 4],
            nodes_trueleafs=[0, 1, 1, 1],
            nodes_falsenodeids=[2, 1, 3, 5],
            nodes_falseleafs=[1, 1, 0, 1],
            leaf_targetids=[0, 0, 0, 0, 0, 0],
            leaf_weights=make_tensor(
                "leaf_weights", input_type, (len(weights),), weights
            ),
        )
        graph = make_graph([node], "ml", [X], [Y])
        return make_model_gen_version(graph, opset_imports=OPSETS)

    @staticmethod
    def _get_test_tree_ensemble_regressor(
        aggregate_function, rule="BRANCH_LEQ", unique_targets=False, base_values=None
    ):
        opsets = [make_opsetid("", TARGET_OPSET), make_opsetid("ai.onnx.ml", 3)]
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        if unique_targets:
            targets = [
                1.0,
                10.0,
                100.0,
                1000.0,
                10000.0,
                100000.0,
            ]
        else:
            targets = [
                0.07692307978868484,
                0.5,
                0.5,
                0.0,
                0.2857142984867096,
                0.5,
            ]
        node1 = make_node(
            "TreeEnsembleRegressor",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            n_targets=1,
            aggregate_function=aggregate_function,
            base_values=base_values,
            nodes_falsenodeids=[4, 3, 0, 0, 0, 2, 0, 4, 0, 0],
            nodes_featureids=[0, 2, 0, 0, 0, 0, 0, 2, 0, 0],
            nodes_hitrates=[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            nodes_missing_value_tracks_true=[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            nodes_modes=[
                rule,
                rule,
                "LEAF",
                "LEAF",
                "LEAF",
                rule,
                "LEAF",
                rule,
                "LEAF",
                "LEAF",
            ],
            nodes_nodeids=[0, 1, 2, 3, 4, 0, 1, 2, 3, 4],
            nodes_treeids=[0, 0, 0, 0, 0, 1, 1, 1, 1, 1],
            nodes_truenodeids=[1, 2, 0, 0, 0, 1, 0, 3, 0, 0],
            nodes_values=[
                0.26645058393478394,
                0.6214364767074585,
                0.0,
                0.0,
                0.0,
                -0.7208403944969177,
                0.0,
                -0.5592705607414246,
                0.0,
                0.0,
            ],
            post_transform="NONE",
            target_ids=[0, 0, 0, 0, 0, 0],
            target_nodeids=[2, 3, 4, 1, 3, 4],
            target_treeids=[0, 0, 0, 1, 1, 1],
            target_weights=targets,
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=opsets)
        onnx.checker.check_model(model)
        return model

    @parameterized.expand(
        tuple(
            itertools.chain.from_iterable(
                (
                    (
                        AggregationFunction.SUM if opset5 else "SUM",
                        np.array(
                            [[0.576923], [0.576923], [0.576923]], dtype=np.float32
                        ),
                        opset5,
                    ),
                    (
                        AggregationFunction.AVERAGE if opset5 else "AVERAGE",
                        np.array(
                            [[0.288462], [0.288462], [0.288462]], dtype=np.float32
                        ),
                        opset5,
                    ),
                    (
                        AggregationFunction.MIN if opset5 else "MIN",
                        np.array(
                            [[0.076923], [0.076923], [0.076923]], dtype=np.float32
                        ),
                        opset5,
                    ),
                    (
                        AggregationFunction.MAX if opset5 else "MAX",
                        np.array([[0.5], [0.5], [0.5]], dtype=np.float32),
                        opset5,
                    ),
                )
                for opset5 in [True, False]
            )
        )
    )
    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_regressor_aggregation_functions(
        self, aggregate_function, expected_result, opset5
    ):
        x = np.arange(9).reshape((-1, 3)).astype(np.float32) / 10 - 0.5
        model_factory = (
            self._get_test_tree_ensemble_opset_latest
            if opset5
            else self._get_test_tree_ensemble_regressor
        )
        model_proto = model_factory(
            aggregate_function,
        )
        sess = ReferenceEvaluator(model_proto)
        (actual,) = sess.run(None, {"X": x})
        assert_allclose(actual, expected_result, atol=1e-6)

    @parameterized.expand(
        tuple(
            itertools.chain.from_iterable(
                (
                    (
                        Mode.LEQ if opset5 else "BRANCH_LEQ",
                        np.array(
                            [[0.576923], [0.576923], [0.576923]], dtype=np.float32
                        ),
                        opset5,
                    ),
                    (
                        Mode.GT if opset5 else "BRANCH_GT",
                        np.array([[0.5], [0.5], [0.5]], dtype=np.float32),
                        opset5,
                    ),
                    (
                        Mode.LT if opset5 else "BRANCH_LT",
                        np.array(
                            [[0.576923], [0.576923], [0.576923]], dtype=np.float32
                        ),
                        opset5,
                    ),
                    (
                        Mode.GTE if opset5 else "BRANCH_GTE",
                        np.array([[0.5], [0.5], [0.5]], dtype=np.float32),
                        opset5,
                    ),
                    (
                        Mode.EQ if opset5 else "BRANCH_EQ",
                        np.array([[1.0], [1.0], [1.0]], dtype=np.float32),
                        opset5,
                    ),
                    (
                        Mode.NEQ if opset5 else "BRANCH_NEQ",
                        np.array(
                            [[0.076923], [0.076923], [0.076923]], dtype=np.float32
                        ),
                        opset5,
                    ),
                )
                for opset5 in [True, False]
            )
        )
    )
    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_regressor_rule(self, rule, expected, opset5):
        x = np.arange(9).reshape((-1, 3)).astype(np.float32) / 10 - 0.5
        model_factory = (
            self._get_test_tree_ensemble_opset_latest
            if opset5
            else self._get_test_tree_ensemble_regressor
        )
        aggregate_function = AggregationFunction.SUM if opset5 else "SUM"

        model_proto = model_factory(aggregate_function, rule)
        sess = ReferenceEvaluator(model_proto)
        (actual,) = sess.run(None, {"X": x})
        assert_allclose(actual, expected, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_regressor_2_targets_opset3(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        opsets = [make_opsetid("", TARGET_OPSET), make_opsetid("ai.onnx.ml", 3)]
        node1 = make_node(
            "TreeEnsembleRegressor",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            n_targets=2,
            nodes_falsenodeids=[4, 3, 0, 0, 6, 0, 0, 4, 3, 0, 0, 6, 0, 0],
            nodes_featureids=[0, 2, 0, 0, 2, 0, 0, 0, 1, 0, 0, 2, 0, 0],
            nodes_hitrates=[
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
            ],
            nodes_missing_value_tracks_true=[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            nodes_modes=[
                "BRANCH_LEQ",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
                "BRANCH_LEQ",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
            ],
            nodes_nodeids=[0, 1, 2, 3, 4, 5, 6, 0, 1, 2, 3, 4, 5, 6],
            nodes_treeids=[0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1],
            nodes_truenodeids=[1, 2, 0, 0, 5, 0, 0, 1, 2, 0, 0, 5, 0, 0],
            nodes_values=[
                -0.3367232382297516,
                1.5326381921768188,
                0.0,
                0.0,
                -0.24646544456481934,
                0.0,
                0.0,
                -0.3367232382297516,
                0.6671845316886902,
                0.0,
                0.0,
                -0.24646544456481934,
                0.0,
                0.0,
            ],
            post_transform="NONE",
            target_ids=[0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1],
            target_nodeids=[2, 2, 3, 3, 5, 5, 6, 6, 2, 2, 3, 3, 5, 5, 6, 6],
            target_treeids=[0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1],
            target_weights=[
                0.0,
                2.5,
                0.5,
                3.0,
                0.15000000596046448,
                2.6500000953674316,
                0.5,
                3.0,
                0.02777777798473835,
                2.527777671813965,
                0.5,
                3.0,
                0.20000000298023224,
                2.700000047683716,
                0.5,
                3.0,
            ],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=opsets)
        onnx.checker.check_model(model)
        x = np.arange(9).reshape((-1, 3)).astype(np.float32) / 10 - 0.5
        expected = np.array(
            [[0.027778, 5.027778], [1.0, 6.0], [1.0, 6.0]], dtype=np.float32
        )
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})
        assert_allclose(got[0], expected, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_regressor_missing_opset3(self):
        x = np.arange(9).reshape((-1, 3)).astype(np.float32) / 10 - 0.5
        x[2, 0] = 5
        x[1, :] = np.nan
        expected = np.array([[100001.0], [100100.0], [100100.0]], dtype=np.float32)
        model = self._get_test_tree_ensemble_regressor("SUM", unique_targets=True)
        self._check_ort(model, {"X": x}, equal=True)
        sess = ReferenceEvaluator(model)
        got = sess.run(None, {"X": x})
        assert_allclose(got[0], expected, atol=1e-6)
        self.assertIn("op_type=TreeEnsembleRegressor", str(sess.rt_nodes_[0]))

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    @parameterized.expand(
        [(input_type,) for input_type in [TensorProto.FLOAT, TensorProto.DOUBLE]]
    )
    def test_tree_ensemble_missing_opset5(self, input_type):
        model = self._get_test_tree_ensemble_opset_latest(
            AggregationFunction.SUM, Mode.LEQ, True, input_type
        )
        np_dtype = onnx.helper.tensor_dtype_to_np_dtype(input_type)
        x = np.arange(9).reshape((-1, 3)).astype(np_dtype) / 10 - 0.5
        x[2, 0] = 5
        x[1, :] = np.nan
        expected = np.array([[100001.0], [100100.0], [100100.0]], dtype=np_dtype)
        session = ReferenceEvaluator(model)
        (actual,) = session.run(None, {"X": x})
        assert_allclose(actual, expected, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_regressor_missing_opset5_float16(self):
        model = self._get_test_tree_ensemble_opset_latest(
            AggregationFunction.SUM, Mode.LEQ, False, TensorProto.FLOAT16
        )
        np_dtype = np.float16
        x = np.arange(9).reshape((-1, 3)).astype(np_dtype) / 10 - 0.5
        x[2, 0] = 5
        x[1, :] = np.nan
        expected = np.array([[0.577], [1.0], [1.0]], dtype=np_dtype)
        session = ReferenceEvaluator(model)
        (actual,) = session.run(None, {"X": x})
        assert_allclose(actual, expected, atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_single_tree_ensemble(self):
        X = make_tensor_value_info("X", TensorProto.DOUBLE, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.DOUBLE, [None, None])
        node = make_node(
            "TreeEnsemble",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            n_targets=2,
            membership_values=None,
            nodes_missing_value_tracks_true=None,
            nodes_hitrates=None,
            aggregate_function=1,
            post_transform=PostTransform.NONE,
            tree_roots=[0],
            nodes_modes=make_tensor(
                "nodes_modes",
                TensorProto.UINT8,
                (3,),
                [Mode.LEQ] * 3,
            ),
            nodes_featureids=[0, 0, 0],
            nodes_splits=make_tensor(
                "nodes_splits",
                TensorProto.DOUBLE,
                (3,),
                np.array([3.14, 1.2, 4.2], dtype=np.float64),
            ),
            nodes_truenodeids=[1, 0, 1],
            nodes_trueleafs=[0, 1, 1],
            nodes_falsenodeids=[2, 2, 3],
            nodes_falseleafs=[0, 1, 1],
            leaf_targetids=[0, 1, 0, 1],
            leaf_weights=make_tensor(
                "leaf_weights",
                TensorProto.DOUBLE,
                (4,),
                np.array([5.23, 12.12, -12.23, 7.21], dtype=np.float64),
            ),
        )
        graph = make_graph([node], "ml", [X], [Y])
        model = make_model_gen_version(
            graph,
            opset_imports=[
                make_opsetid("", TARGET_OPSET),
                make_opsetid("ai.onnx.ml", 5),
            ],
        )
        onnx.checker.check_model(model)
        session = ReferenceEvaluator(model)
        (output,) = session.run(
            None,
            {
                "X": np.array([1.2, 3.4, -0.12, 1.66, 4.14, 1.77], np.float64).reshape(
                    3, 2
                )
            },
        )
        np.testing.assert_equal(
            output, np.array([[5.23, 0], [5.23, 0], [0, 12.12]], dtype=np.float64)
        )

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_regressor_set_membership_opset5(self):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node = make_node(
            "TreeEnsemble",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            n_targets=4,
            aggregate_function=AggregationFunction.SUM,
            membership_values=make_tensor(
                "membership_values",
                TensorProto.FLOAT,
                (8,),
                [1.2, 3.7, 8, 9, np.nan, 12, 7, np.nan],
            ),
            nodes_missing_value_tracks_true=None,
            nodes_hitrates=None,
            post_transform=PostTransform.NONE,
            tree_roots=[0],
            nodes_modes=make_tensor(
                "nodes_modes",
                TensorProto.UINT8,
                (3,),
                [Mode.LEQ, Mode.MEMBER, Mode.MEMBER],
            ),
            nodes_featureids=[0, 0, 0],
            nodes_splits=make_tensor(
                "nodes_splits",
                TensorProto.FLOAT,
                (3,),
                np.array([11, 232344.0, np.nan], dtype=np.float32),
            ),
            nodes_trueleafs=[0, 1, 1],
            nodes_truenodeids=[1, 0, 1],
            nodes_falseleafs=[1, 0, 1],
            nodes_falsenodeids=[2, 2, 3],
            leaf_targetids=[0, 1, 2, 3],
            leaf_weights=make_tensor(
                "leaf_weights", TensorProto.FLOAT, (4,), [1, 10, 1000, 100]
            ),
        )
        graph = make_graph([node], "ml", [X], [Y])
        model = make_model_gen_version(
            graph,
            opset_imports=OPSETS,
        )
        onnx.checker.check_model(model)
        session = ReferenceEvaluator(model)
        X = np.array([1.2, 3.4, -0.12, np.nan, 12, 7], np.float32).reshape(-1, 1)
        expected = np.array(
            [
                [1, 0, 0, 0],
                [0, 0, 0, 100],
                [0, 0, 0, 100],
                [0, 0, 1000, 0],
                [0, 0, 1000, 0],
                [0, 10, 0, 0],
            ],
            dtype=np.float32,
        )
        (output,) = session.run(None, {"X": X})
        np.testing.assert_equal(output, expected)

    @staticmethod
    def _get_test_svm_regressor(kernel_type, kernel_params):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "SVMRegressor",
            ["X"],
            ["Y"],
            domain="ai.onnx.ml",
            coefficients=[
                1.0,
                -1.0,
                0.8386201858520508,
                -0.8386201858520508,
                0.4470679759979248,
                -1.0,
                0.5529320240020752,
            ],
            kernel_params=kernel_params,
            kernel_type=kernel_type,
            n_supports=7,
            post_transform="NONE",
            rho=[0.5460880398750305],
            support_vectors=[
                -0.12850627303123474,
                0.08915442228317261,
                0.06881910562515259,
                -0.07938569784164429,
                -0.22557435929775238,
                -0.26520243287086487,
                0.9246066212654114,
                -0.025557516142725945,
                -0.5900523662567139,
                0.9735698699951172,
                -1.3385062217712402,
                0.3393094539642334,
                0.9432410001754761,
                -0.5228781700134277,
                0.5557093620300293,
                0.4191802740097046,
                0.43368014693260193,
                -1.0569839477539062,
                2.3318440914154053,
                0.06202844902873039,
                -0.9502395987510681,
            ],
        )
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        return model

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_regressor(self):
        x = np.arange(9).reshape((-1, 3)).astype(np.float32) / 10 - 0.5
        expected_kernel = {
            "LINEAR": (
                [0.42438405752182007, 0.0, 3.0],
                np.array([[-0.468206], [0.227487], [0.92318]], dtype=np.float32),
            ),
            "POLY": (
                [0.3426632285118103, 0.0, 3.0],
                np.array([[0.527084], [0.543578], [0.546506]], dtype=np.float32),
            ),
            "RBF": (
                [0.30286383628845215, 0.0, 3.0],
                np.array([[0.295655], [0.477876], [0.695292]], dtype=np.float32),
            ),
            "SIGMOID": (
                [0.30682486295700073, 0.0, 3.0],
                np.array([[0.239304], [0.448929], [0.661689]], dtype=np.float32),
            ),
        }
        for kernel, (params, expected) in expected_kernel.items():
            with self.subTest(kernel=kernel):
                model = self._get_test_svm_regressor(kernel, params)
                self._check_ort(model, {"X": x}, atol=1e-6)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[0], expected, atol=1e-6)

    @staticmethod
    def _get_test_tree_ensemble_classifier_binary(post_transform):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        In = make_tensor_value_info("I", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "TreeEnsembleClassifier",
            ["X"],
            ["I", "Y"],
            domain="ai.onnx.ml",
            class_ids=[0, 0, 0, 0, 0, 0, 0],
            class_nodeids=[2, 3, 5, 6, 1, 3, 4],
            class_treeids=[0, 0, 0, 0, 1, 1, 1],
            class_weights=[
                0.0,
                0.1764705926179886,
                0.0,
                0.5,
                0.0,
                0.0,
                0.4285714328289032,
            ],
            classlabels_int64s=[0, 1],
            nodes_falsenodeids=[4, 3, 0, 0, 6, 0, 0, 2, 0, 4, 0, 0],
            nodes_featureids=[2, 2, 0, 0, 1, 0, 0, 2, 0, 0, 0, 0],
            nodes_hitrates=[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            nodes_missing_value_tracks_true=[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            nodes_modes=[
                "BRANCH_LEQ",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
                "BRANCH_LEQ",
                "LEAF",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
            ],
            nodes_nodeids=[0, 1, 2, 3, 4, 5, 6, 0, 1, 2, 3, 4],
            nodes_treeids=[0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1],
            nodes_truenodeids=[1, 2, 0, 0, 5, 0, 0, 1, 0, 3, 0, 0],
            nodes_values=[
                0.6874135732650757,
                -0.3654803931713104,
                0.0,
                0.0,
                -1.926770806312561,
                0.0,
                0.0,
                -0.3654803931713104,
                0.0,
                -2.0783839225769043,
                0.0,
                0.0,
            ],
            post_transform=post_transform,
        )
        graph = make_graph([node1], "ml", [X], [In, Y])
        model = make_model_gen_version(
            graph,
            opset_imports=[
                make_opsetid("", TARGET_OPSET),
                make_opsetid("ai.onnx.ml", 3),
            ],
        )
        onnx.checker.check_model(model)
        return model

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_classifier_binary(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        expected_post = {
            "NONE": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[1.0, 0.0], [0.394958, 0.605042], [0.394958, 0.605042]],
                    dtype=np.float32,
                ),
            ),
            "LOGISTIC": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.5, 0.5], [0.353191, 0.646809], [0.353191, 0.646809]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.5, 0.5], [0.229686, 0.770314], [0.229686, 0.770314]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX_ZERO": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.5, 0.5], [0.229686, 0.770314], [0.229686, 0.770314]],
                    dtype=np.float32,
                ),
            ),
            "PROBIT": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.0, 0.0], [-0.266426, 0.266426], [-0.266426, 0.266426]],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_tree_ensemble_classifier_binary(post)
                if post in ("NONE",):
                    self._check_ort(model, {"X": x})
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @staticmethod
    def _get_test_tree_ensemble_classifier_multi(post_transform):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        In = make_tensor_value_info("I", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        node1 = make_node(
            "TreeEnsembleClassifier",
            ["X"],
            ["I", "Y"],
            domain="ai.onnx.ml",
            class_ids=[0, 1, 2, 0, 1, 2, 0, 1, 2, 0, 1, 2, 0, 1, 2, 0, 1, 2],
            class_nodeids=[2, 2, 2, 3, 3, 3, 4, 4, 4, 1, 1, 1, 3, 3, 3, 4, 4, 4],
            class_treeids=[0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1],
            class_weights=[
                0.46666666865348816,
                0.0,
                0.03333333507180214,
                0.20000000298023224,
                0.23999999463558197,
                0.05999999865889549,
                0.0,
                0.5,
                0.0,
                0.5,
                0.0,
                0.0,
                0.44999998807907104,
                0.0,
                0.05000000074505806,
                0.10294117778539658,
                0.19117647409439087,
                0.20588235557079315,
            ],
            classlabels_int64s=[0, 1, 2],
            nodes_falsenodeids=[4, 3, 0, 0, 0, 2, 0, 4, 0, 0],
            nodes_featureids=[1, 0, 0, 0, 0, 1, 0, 0, 0, 0],
            nodes_hitrates=[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            nodes_missing_value_tracks_true=[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            nodes_modes=[
                "BRANCH_LEQ",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
                "LEAF",
                "BRANCH_LEQ",
                "LEAF",
                "BRANCH_LEQ",
                "LEAF",
                "LEAF",
            ],
            nodes_nodeids=[0, 1, 2, 3, 4, 0, 1, 2, 3, 4],
            nodes_treeids=[0, 0, 0, 0, 0, 1, 1, 1, 1, 1],
            nodes_truenodeids=[1, 2, 0, 0, 0, 1, 0, 3, 0, 0],
            nodes_values=[
                1.2495747804641724,
                -0.3050493597984314,
                0.0,
                0.0,
                0.0,
                -1.6830512285232544,
                0.0,
                -0.6751254796981812,
                0.0,
                0.0,
            ],
            post_transform=post_transform,
        )
        graph = make_graph([node1], "ml", [X], [In, Y])
        model = make_model_gen_version(
            graph,
            opset_imports=[
                make_opsetid("", TARGET_OPSET),
                make_opsetid("ai.onnx.ml", 3),
            ],
        )
        onnx.checker.check_model(model)
        return model

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_tree_ensemble_classifier_multi(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        expected_post = {
            "NONE": (
                np.array([0, 0, 1], dtype=np.int64),
                np.array(
                    [
                        [0.916667, 0.0, 0.083333],
                        [0.569608, 0.191176, 0.239216],
                        [0.302941, 0.431176, 0.265882],
                    ],
                    dtype=np.float32,
                ),
            ),
            "LOGISTIC": (
                np.array([0, 0, 1], dtype=np.int64),
                np.array(
                    [
                        [0.714362, 0.5, 0.520821],
                        [0.638673, 0.547649, 0.55952],
                        [0.575161, 0.606155, 0.566082],
                    ],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX": (
                np.array([0, 0, 1], dtype=np.int64),
                np.array(
                    [
                        [0.545123, 0.217967, 0.23691],
                        [0.416047, 0.284965, 0.298988],
                        [0.322535, 0.366664, 0.310801],
                    ],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX_ZERO": (
                np.array([0, 0, 1], dtype=np.int64),
                np.array(
                    [
                        [0.697059, 0.0, 0.302941],
                        [0.416047, 0.284965, 0.298988],
                        [0.322535, 0.366664, 0.310801],
                    ],
                    dtype=np.float32,
                ),
            ),
            "PROBIT": (
                np.array([0, 0, 1], dtype=np.int64),
                np.array(
                    [
                        [1.383104, 0, -1.383105],
                        [0.175378, -0.873713, -0.708922],
                        [-0.516003, -0.173382, -0.625385],
                    ],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_tree_ensemble_classifier_multi(post)
                if post != "PROBIT":
                    self._check_ort(model, {"X": x}, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @staticmethod
    def _get_test_svm_classifier_binary(post_transform, probability=True, linear=False):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        In = make_tensor_value_info("I", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        if linear:
            kwargs = {
                "classlabels_ints": [0, 1, 2, 3],
                "coefficients": [
                    -1.55181212e-01,
                    2.42698956e-01,
                    7.01893432e-03,
                    4.07614474e-01,
                    -3.24927823e-02,
                    2.79897536e-04,
                    -1.95771302e-01,
                    -3.52437368e-01,
                    -2.15973096e-02,
                    -4.38190277e-01,
                    4.56869105e-02,
                    -1.29375499e-02,
                ],
                "kernel_params": [0.001, 0.0, 3.0],
                "kernel_type": "LINEAR",
                "prob_a": [-5.139118194580078],
                "prob_b": [0.06399919837713242],
                "rho": [-0.07489691, -0.1764396, -0.21167431, -0.51619097],
                "post_transform": post_transform,
            }
        else:
            kwargs = {
                "classlabels_ints": [0, 1],
                "coefficients": [1.0, 1.0, 1.0, 1.0, -1.0, -1.0, -1.0, -1.0],
                "kernel_params": [0.3824487328529358, 0.0, 3.0],
                "kernel_type": "RBF",
                "prob_a": [-5.139118194580078],
                "prob_b": [0.06399919837713242],
                "rho": [0.16708599030971527],
                "support_vectors": [
                    0.19125767052173615,
                    -1.062204122543335,
                    0.5006636381149292,
                    -0.5892484784126282,
                    -0.3196830451488495,
                    0.0984845906496048,
                    0.24746321141719818,
                    -1.1535362005233765,
                    0.4109955430030823,
                    -0.5937694907188416,
                    -1.3183348178863525,
                    -1.6423596143722534,
                    0.558641254901886,
                    -0.9218668341636658,
                    0.6264089345932007,
                    -0.16060839593410492,
                    -0.6365169882774353,
                    0.8335472345352173,
                    0.7539799213409424,
                    -0.3970031440258026,
                    -0.1780400276184082,
                    -0.616622805595398,
                    0.49261474609375,
                    0.4470972716808319,
                ],
                "vectors_per_class": [4, 4],
                "post_transform": post_transform,
            }

        if not probability:
            del kwargs["prob_a"]
            del kwargs["prob_b"]
        node1 = make_node(
            "SVMClassifier", ["X"], ["I", "Y"], domain="ai.onnx.ml", **kwargs
        )
        graph = make_graph([node1], "ml", [X], [In, Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        return model

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_classifier_binary(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        expected_post = {
            "NONE": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.993287, 0.006713], [0.469401, 0.530599], [0.014997, 0.985003]],
                    dtype=np.float32,
                ),
            ),
            "LOGISTIC": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.729737, 0.501678], [0.615242, 0.629623], [0.503749, 0.7281]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.728411, 0.271589], [0.484705, 0.515295], [0.274879, 0.725121]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX_ZERO": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.728411, 0.271589], [0.484705, 0.515295], [0.274879, 0.725121]],
                    dtype=np.float32,
                ),
            ),
            "PROBIT": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[2.469393, -2.469391], [-0.076776, 0.076776], [-2.16853, 2.16853]],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_svm_classifier_binary(post)
                self._check_ort(model, {"X": x}, rev=True, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-5)
                assert_allclose(got[0], expected[0])

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_classifier_binary_noprob(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        expected_post = {
            "NONE": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [
                        [-0.986073, 0.986073],
                        [0.011387, -0.011387],
                        [0.801808, -0.801808],
                    ],
                    dtype=np.float32,
                ),
            ),
            "LOGISTIC": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.271688, 0.728312], [0.502847, 0.497153], [0.690361, 0.309639]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.122158, 0.877842], [0.505693, 0.494307], [0.832523, 0.167477]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX_ZERO": (
                np.array([0, 1, 1], dtype=np.int64),
                np.array(
                    [[0.122158, 0.877842], [0.505693, 0.494307], [0.832523, 0.167477]],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_svm_classifier_binary(post, probability=False)
                if post not in {"LOGISTIC", "SOFTMAX", "SOFTMAX_ZERO", "PROBIT"}:
                    self._check_ort(model, {"X": x}, rev=True, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_classifier_noprob_linear(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        nan = np.nan
        expected_post = {
            "NONE": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [-0.118086, -0.456685, 0.415783, 0.334506],
                        [-0.061364, -0.231444, 0.073899, 0.091242],
                        [-0.004642, -0.006203, -0.267985, -0.152023],
                    ],
                    dtype=np.float32,
                ),
            ),
            "LOGISTIC": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [0.470513, 0.387773, 0.602474, 0.582855],
                        [0.484664, 0.442396, 0.518466, 0.522795],
                        [0.498839, 0.498449, 0.433402, 0.462067],
                    ],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [0.200374, 0.14282, 0.341741, 0.315065],
                        [0.240772, 0.203115, 0.275645, 0.280467],
                        [0.275491, 0.275061, 0.211709, 0.237739],
                    ],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX_ZERO": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [0.200374, 0.14282, 0.341741, 0.315065],
                        [0.240772, 0.203115, 0.275645, 0.280467],
                        [0.275491, 0.275061, 0.211709, 0.237739],
                    ],
                    dtype=np.float32,
                ),
            ),
            "PROBIT": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [nan, nan, -0.212698, -0.427529],
                        [nan, nan, -1.447414, -1.333286],
                        [nan, nan, nan, nan],
                    ],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_svm_classifier_binary(
                    post, probability=False, linear=True
                )
                self._check_ort(model, {"X": x}, rev=True, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_classifier_linear(self):
        # prob_a, prob_b are not used in this case.
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        nan = np.nan
        expected_post = {
            "NONE": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [-0.118086, -0.456685, 0.415783, 0.334506],
                        [-0.061364, -0.231444, 0.073899, 0.091242],
                        [-0.004642, -0.006203, -0.267985, -0.152023],
                    ],
                    dtype=np.float32,
                ),
            ),
            "LOGISTIC": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [0.470513, 0.387773, 0.602474, 0.582855],
                        [0.484664, 0.442396, 0.518466, 0.522795],
                        [0.498839, 0.498449, 0.433402, 0.462067],
                    ],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [0.200374, 0.14282, 0.341741, 0.315065],
                        [0.240772, 0.203115, 0.275645, 0.280467],
                        [0.275491, 0.275061, 0.211709, 0.237739],
                    ],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX_ZERO": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [0.200374, 0.14282, 0.341741, 0.315065],
                        [0.240772, 0.203115, 0.275645, 0.280467],
                        [0.275491, 0.275061, 0.211709, 0.237739],
                    ],
                    dtype=np.float32,
                ),
            ),
            "PROBIT": (
                np.array([2, 3, 0], dtype=np.int64),
                np.array(
                    [
                        [nan, nan, -0.212698, -0.427529],
                        [nan, nan, -1.447414, -1.333286],
                        [nan, nan, nan, nan],
                    ],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_svm_classifier_binary(
                    post, probability=True, linear=True
                )
                self._check_ort(model, {"X": x}, rev=True, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @staticmethod
    def _get_test_svm_classifier_linear_sv(post_transform, probability=True):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        In = make_tensor_value_info("I", TensorProto.INT64, [None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None, None])
        kwargs = {
            "classlabels_ints": [0, 1],
            "coefficients": [
                0.766398549079895,
                0.0871576070785522,
                0.110420741140842,
                -0.963976919651031,
            ],
            "support_vectors": [
                4.80000019073486,
                3.40000009536743,
                1.89999997615814,
                5.0,
                3.0,
                1.60000002384186,
                4.5,
                2.29999995231628,
                1.29999995231628,
                5.09999990463257,
                2.5,
                3.0,
            ],
            "kernel_params": [0.122462183237076, 0.0, 3.0],
            "kernel_type": "LINEAR",
            "prob_a": [-5.139118194580078],
            "prob_b": [0.06399919837713242],
            "rho": [2.23510527610779],
            "post_transform": post_transform,
            "vectors_per_class": [3, 1],
        }

        if not probability:
            del kwargs["prob_a"]
            del kwargs["prob_b"]
        node1 = make_node(
            "SVMClassifier", ["X"], ["I", "Y"], domain="ai.onnx.ml", **kwargs
        )
        graph = make_graph([node1], "ml", [X], [In, Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        return model

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_classifier_binary_noprob_linear_sv(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        expected_post = {
            "NONE": (
                np.array([0, 0, 0], dtype=np.int64),
                np.array(
                    [[-2.662655, 2.662655], [-2.21481, 2.21481], [-1.766964, 1.766964]],
                    dtype=np.float32,
                ),
            ),
            "LOGISTIC": (
                np.array([0, 0, 0], dtype=np.int64),
                np.array(
                    [[0.065213, 0.934787], [0.098428, 0.901572], [0.14592, 0.85408]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX": (
                np.array([0, 0, 0], dtype=np.int64),
                np.array(
                    [[0.004843, 0.995157], [0.011779, 0.988221], [0.028362, 0.971638]],
                    dtype=np.float32,
                ),
            ),
            "SOFTMAX_ZERO": (
                np.array([0, 0, 0], dtype=np.int64),
                np.array(
                    [[0.004843, 0.995157], [0.011779, 0.988221], [0.028362, 0.971638]],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_svm_classifier_linear_sv(post, probability=False)
                if post not in {"LOGISTIC", "SOFTMAX", "SOFTMAX_ZERO"}:
                    self._check_ort(model, {"X": x}, rev=True, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[1], expected[1], atol=1e-6)
                assert_allclose(got[0], expected[0])

    @staticmethod
    def _get_test_svm_regressor_linear(post_transform, one_class=0):
        X = make_tensor_value_info("X", TensorProto.FLOAT, [None, None])
        Y = make_tensor_value_info("Y", TensorProto.FLOAT, [None])
        kwargs = {
            "coefficients": [0.28290501, -0.0266512, 0.01674867],
            "kernel_params": [0.001, 0.0, 3.0],
            "kernel_type": "LINEAR",
            "rho": [1.24032312],
            "post_transform": post_transform,
            "n_supports": 0,
            "one_class": one_class,
        }

        node1 = make_node("SVMRegressor", ["X"], ["Y"], domain="ai.onnx.ml", **kwargs)
        graph = make_graph([node1], "ml", [X], [Y])
        model = make_model_gen_version(graph, opset_imports=OPSETS)
        onnx.checker.check_model(model)
        return model

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_regressor_linear(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        expected_post = {
            "NONE": (
                np.array(
                    [[0.96869], [1.132491], [1.296293]],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_svm_regressor_linear(post)
                self._check_ort(model, {"X": x}, rev=True, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[0], expected[0], atol=1e-6)

    @unittest.skipIf(not ONNX_ML, reason="onnx not compiled with ai.onnx.ml")
    def test_svm_regressor_linear_one_class(self):
        x = (np.arange(9).reshape((-1, 3)) - 5).astype(np.float32) / 5
        expected_post = {
            "NONE": (
                np.array(
                    [[1.0], [1.0], [1.0]],
                    dtype=np.float32,
                ),
            ),
        }
        for post, expected in expected_post.items():
            with self.subTest(post_transform=post):
                model = self._get_test_svm_regressor_linear(post, one_class=1)
                self._check_ort(model, {"X": x}, rev=True, atol=1e-5)
                sess = ReferenceEvaluator(model)
                got = sess.run(None, {"X": x})
                assert_allclose(got[0], expected[0], atol=1e-6)

    def test_onnxrt_tfidf_vectorizer_ints(self):
        inputi = np.array([[1, 1, 3, 3, 3, 7], [8, 6, 7, 5, 6, 8]]).astype(np.int64)
        output = np.array(
            [[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0]]
        ).astype(np.float32)

        ngram_counts = np.array([0, 4]).astype(np.int64)
        ngram_indexes = np.array([0, 1, 2, 3, 4, 5, 6]).astype(np.int64)
        pool_int64s = np.array([2, 3, 5, 4, 5, 6, 7, 8, 6, 7]).astype(  # unigrams
            np.int64
        )  # bigrams

        model = make_model_gen_version(
            make_graph(
                [
                    make_node(
                        "TfIdfVectorizer",
                        ["tokens"],
                        ["out"],
                        mode="TF",
                        min_gram_length=2,
                        max_gram_length=2,
                        max_skip_count=0,
                        ngram_counts=ngram_counts,
                        ngram_indexes=ngram_indexes,
                        pool_int64s=pool_int64s,
                    )
                ],
                "tfidf",
                [make_tensor_value_info("tokens", TensorProto.INT64, [None, None])],
                [make_tensor_value_info("out", TensorProto.FLOAT, [None, None])],
            ),
            opset_imports=OPSETS,
        )

        oinf = ReferenceEvaluator(model)
        res = oinf.run(None, {"tokens": inputi})
        self.assertEqual(output.tolist(), res[0].tolist())

    def test_onnxrt_tfidf_vectorizer_strings(self):
        inputi = np.array(
            [["i1", "i1", "i3", "i3", "i3", "i7"], ["i8", "i6", "i7", "i5", "i6", "i8"]]
        )
        output = np.array(
            [[0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0]]
        ).astype(np.float32)

        ngram_counts = np.array([0, 4]).astype(np.int64)
        ngram_indexes = np.array([0, 1, 2, 3, 4, 5, 6]).astype(np.int64)
        pool_strings = np.array(
            ["i2", "i3", "i5", "i4", "i5", "i6", "i7", "i8", "i6", "i7"]
        )

        model = make_model_gen_version(
            make_graph(
                [
                    make_node(
                        "TfIdfVectorizer",
                        ["tokens"],
                        ["out"],
                        mode="TF",
                        min_gram_length=2,
                        max_gram_length=2,
                        max_skip_count=0,
                        ngram_counts=ngram_counts,
                        ngram_indexes=ngram_indexes,
                        pool_strings=pool_strings,
                    )
                ],
                "tfidf",
                [make_tensor_value_info("tokens", TensorProto.INT64, [None, None])],
                [make_tensor_value_info("out", TensorProto.FLOAT, [None, None])],
            ),
            opset_imports=OPSETS,
        )

        oinf = ReferenceEvaluator(model)
        res = oinf.run(None, {"tokens": inputi})
        self.assertEqual(output.tolist(), res[0].tolist())


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import os
import tarfile
from collections import deque
from typing import TYPE_CHECKING

import onnx.checker
import onnx.helper
import onnx.shape_inference

if TYPE_CHECKING:
    from onnx.onnx_pb import (
        FunctionProto,
        ModelProto,
        NodeProto,
        TensorProto,
        ValueInfoProto,
    )


class Extractor:
    def __init__(self, model: ModelProto) -> None:
        self.model = model
        self.graph = self.model.graph
        self.initializers: dict[str, TensorProto] = self._build_name2obj_dict(
            self.graph.initializer
        )
        self.value_infos: dict[str, ValueInfoProto] = self._build_name2obj_dict(
            self.graph.value_info
        )
        # Add input and output values (not included in the value_info for intermediate values)
        self.value_infos.update(self._build_name2obj_dict(self.graph.input))
        self.value_infos.update(self._build_name2obj_dict(self.graph.output))
        self.outmap: dict[str, int] = self._build_output_dict(self.graph)

    @staticmethod
    def _build_name2obj_dict(objs) -> dict:
        return {obj.name: obj for obj in objs}

    @staticmethod
    def _build_output_dict(graph) -> dict[str, int]:
        output_to_index: dict[str, int] = {}
        for index, node in enumerate(graph.node):
            for output_name in node.output:
                if output_name == "":
                    continue
                assert output_name not in output_to_index  # output_name is unique
                output_to_index[output_name] = index
        return output_to_index

    def _collect_new_io(self, io_names_to_extract: list[str]) -> list[ValueInfoProto]:
        # Validate that all names exist in self.value_infos
        missing_names = [
            name for name in io_names_to_extract if name not in self.value_infos
        ]
        if missing_names:
            raise ValueError(
                f"The following names were not found in value_infos: {', '.join(missing_names)}"
            )
        return [self.value_infos[name] for name in io_names_to_extract]

    def _dfs_search_reachable_nodes(
        self,
        node_output_name: str,
        graph_input_names: set[str],
        reachable: set[int],
    ) -> None:
        """Helper function to find nodes which are connected to an output

        Arguments:
            node_output_name (str): The name of the output
            graph_input_names (set of string): The names of all inputs of the graph
            reachable (set of int): The set of indexes to reachable nodes in `nodes`
        """
        stack = [node_output_name]
        while stack:
            current_output_name = stack.pop()
            # finish search at inputs
            if current_output_name in graph_input_names:
                continue
            # find nodes connected to this output
            if current_output_name in self.outmap:
                index = self.outmap[current_output_name]
                if index not in reachable:
                    # add nodes connected to this output to sets
                    reachable.add(index)
                    stack += [
                        input_name
                        for input_name in self.graph.node[index].input
                        if input_name != ""
                    ]

    def _collect_reachable_nodes(
        self,
        input_names: list[str],
        output_names: list[str],
    ) -> list[NodeProto]:
        _input_names = set(input_names)
        reachable: set[int] = set()
        for name in output_names:
            self._dfs_search_reachable_nodes(name, _input_names, reachable)
        # needs to be topologically sorted
        return [self.graph.node[index] for index in sorted(reachable)]

    def _collect_referred_local_functions(
        self,
        nodes: list[NodeProto],
    ) -> list[FunctionProto]:
        # a node in a model graph may refer a function.
        # a function contains nodes, some of which may in turn refer a function.
        # we need to find functions referred by graph nodes and
        # by nodes used to define functions.
        function_map: dict[tuple[str, str], FunctionProto] = {}
        for function in self.model.functions:
            function_map[(function.name, function.domain)] = function
        referred_local_functions: list[FunctionProto] = []
        queue = deque(nodes)
        while queue:
            node = queue.popleft()
            # check if the node is a function op
            if (node.op_type, node.domain) in function_map:
                function = function_map.pop((node.op_type, node.domain))
                referred_local_functions.append(function)
                queue.extend(function.node)
        # needs to be topologically sorted
        return referred_local_functions

    def _collect_reachable_tensors(
        self,
        nodes: list[NodeProto],
    ) -> tuple[list[TensorProto], list[ValueInfoProto]]:
        all_tensors_names: set[str] = set()
        for node in nodes:
            all_tensors_names.update(node.input)
            all_tensors_names.update(node.output)
        initializer = [
            self.initializers[t] for t in self.initializers if t in all_tensors_names
        ]
        value_info = [
            self.value_infos[t] for t in self.value_infos if t in all_tensors_names
        ]
        len_sparse_initializer = len(self.graph.sparse_initializer)
        if len_sparse_initializer != 0:
            raise ValueError(
                f"len_sparse_initializer is {len_sparse_initializer}, it must be 0."
            )
        len_quantization_annotation = len(self.graph.quantization_annotation)
        if len_quantization_annotation != 0:
            raise ValueError(
                f"len_quantization_annotation is {len_quantization_annotation}, it must be 0."
            )
        return initializer, value_info

    def _make_model(
        self,
        nodes: list[NodeProto],
        inputs: list[ValueInfoProto],
        outputs: list[ValueInfoProto],
        initializer: list[TensorProto],
        value_info: list[ValueInfoProto],
        local_functions: list[FunctionProto],
    ) -> ModelProto:
        name = "Extracted from {" + self.graph.name + "}"
        graph = onnx.helper.make_graph(
            nodes, name, inputs, outputs, initializer=initializer, value_info=value_info
        )
        meta = {
            "ir_version": self.model.ir_version,
            "opset_imports": self.model.opset_import,
            "producer_name": "onnx.utils.extract_model",
            "functions": local_functions,
        }
        return onnx.helper.make_model(graph, **meta)

    def extract_model(
        self,
        input_names: list[str],
        output_names: list[str],
    ) -> ModelProto:
        inputs = self._collect_new_io(input_names)
        outputs = self._collect_new_io(output_names)
        nodes = self._collect_reachable_nodes(input_names, output_names)
        initializer, value_info = self._collect_reachable_tensors(nodes)
        local_functions = self._collect_referred_local_functions(nodes)
        return self._make_model(
            nodes, inputs, outputs, initializer, value_info, local_functions
        )


def extract_model(
    input_path: str | os.PathLike,
    output_path: str | os.PathLike,
    input_names: list[str],
    output_names: list[str],
    check_model: bool = True,
    infer_shapes: bool = True,
) -> None:
    """Extracts sub-model from an ONNX model.

    The sub-model is defined by the names of the input and output tensors *exactly*.

    Note: For control-flow operators, e.g. If and Loop, the _boundary of sub-model_,
    which is defined by the input and output tensors, should not _cut through_ the
    subgraph that is connected to the _main graph_ as attributes of these operators.

    Note: When the extracted model size is larger than 2GB, the extra data will be saved in "output_path.data".

    Arguments:
        input_path (str | os.PathLike): The path to original ONNX model.
        output_path (str | os.PathLike): The path to save the extracted ONNX model.
        input_names (list of string): The names of the input tensors that to be extracted.
        output_names (list of string): The names of the output tensors that to be extracted.
        check_model (bool): Whether to run model checker on the original model and the extracted model.
        infer_shapes (bool): Whether to infer the shapes of the original model.
    """
    if not os.path.exists(input_path):
        raise ValueError(f"Invalid input model path: {input_path}")
    if not output_path:
        raise ValueError("Output model path shall not be empty!")
    if not input_names:
        raise ValueError("Input tensor names shall not be empty!")
    if not output_names:
        raise ValueError("Output tensor names shall not be empty!")

    if len(input_names) != len(set(input_names)):
        raise ValueError("Duplicate names found in the input tensor names.")
    if len(output_names) != len(set(output_names)):
        raise ValueError("Duplicate names found in the output tensor names.")

    if check_model:
        onnx.checker.check_model(input_path)

    if infer_shapes and os.path.getsize(input_path) > onnx.checker.MAXIMUM_PROTOBUF:
        onnx.shape_inference.infer_shapes_path(input_path, output_path)
        model = onnx.load(output_path)
    elif infer_shapes:
        model = onnx.load(input_path, load_external_data=False)
        model = onnx.shape_inference.infer_shapes(model)
        base_dir = os.path.dirname(input_path)
        onnx.load_external_data_for_model(model, base_dir)
    else:
        model = onnx.load(input_path)

    e = Extractor(model)
    extracted = e.extract_model(input_names, output_names)

    if extracted.ByteSize() > onnx.checker.MAXIMUM_PROTOBUF:
        location = os.path.basename(output_path) + ".data"
        onnx.save(extracted, output_path, save_as_external_data=True, location=location)
    else:
        onnx.save(extracted, output_path)

    if check_model:
        onnx.checker.check_model(output_path)


def _tar_members_filter(
    tar: tarfile.TarFile, base: str | os.PathLike
) -> list[tarfile.TarInfo]:
    """Check that the content of ``tar`` will be extracted safely

    Args:
        tar: The tarball file
        base: The directory where the tarball will be extracted

    Returns:
        list of tarball members
    """
    result = []
    for member in tar:
        member_path = os.path.join(base, member.name)
        abs_base = os.path.abspath(base)
        abs_member = os.path.abspath(member_path)
        if not abs_member.startswith(abs_base):
            raise RuntimeError(
                f"The tarball member {member_path} in downloading model contains "
                f"directory traversal sequence which may contain harmful payload."
            )
        if member.issym() or member.islnk():
            raise RuntimeError(
                f"The tarball member {member_path} in downloading model contains "
                f"symbolic links which may contain harmful payload."
            )
        result.append(member)
    return result


def _extract_model_safe(
    model_tar_path: str | os.PathLike, local_model_with_data_dir_path: str | os.PathLike
) -> None:
    """Safely extracts a tar file to a specified directory.

    This function ensures that the extraction process mitigates against
    directory traversal vulnerabilities by validating or sanitizing paths
    within the tar file. It also provides compatibility for different versions
    of the tarfile module by checking for the availability of certain attributes
    or methods before invoking them.

    Args:
        model_tar_path: The path to the tar file to be extracted.
        local_model_with_data_dir_path: The directory path where the tar file
      contents will be extracted to.
    """
    with tarfile.open(model_tar_path) as model_with_data_zipped:
        # Mitigate tarball directory traversal risks
        if hasattr(tarfile, "data_filter"):
            model_with_data_zipped.extractall(
                path=local_model_with_data_dir_path, filter="data"
            )
        else:
            model_with_data_zipped.extractall(
                path=local_model_with_data_dir_path,
                members=_tar_members_filter(
                    model_with_data_zipped, local_model_with_data_dir_path
                ),
            )



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.ops.aionnxml._op_run_aionnxml import OpRunAiOnnxMl


class DictVectorizer(OpRunAiOnnxMl):
    def _run(self, x, int64_vocabulary=None, string_vocabulary=None):
        if isinstance(x, (np.ndarray, list)):
            if int64_vocabulary is None and string_vocabulary is None:
                raise ValueError(
                    "int64_vocabulary or string_vocabulary must be provided."
                )
            if int64_vocabulary is not None:
                dict_labels = {v: i for i, v in enumerate(int64_vocabulary)}
            else:
                assert string_vocabulary is not None
                dict_labels = {v: i for i, v in enumerate(string_vocabulary)}

            values_list = []
            rows_list = []
            cols_list = []
            for i, row in enumerate(x):
                for k, v in row.items():
                    values_list.append(v)
                    rows_list.append(i)
                    cols_list.append(dict_labels[k])
            values = np.array(values_list)
            rows = np.array(rows_list)
            cols = np.array(cols_list)

            res = np.zeros((len(x), len(dict_labels)), dtype=values.dtype)
            for r, c, v in zip(rows, cols, values, strict=False):
                res[r, c] = v
            return (res,)

        if isinstance(x, dict):
            keys = int64_vocabulary or string_vocabulary
            assert keys is not None
            result = [x.get(k, 0) for k in keys]
            return (np.array(result),)

        raise TypeError(f"x must be iterable not {type(x)}.")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: onnx/onnx-ml.proto
# Protobuf Python Version: 7.34.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    7,
    34,
    1,
    '',
    'onnx/onnx-ml.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12onnx/onnx-ml.proto\x12\x04onnx\"\xdb\x05\n\x0e\x41ttributeProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rref_attr_name\x18\x15 \x01(\t\x12\x12\n\ndoc_string\x18\r \x01(\t\x12\x30\n\x04type\x18\x14 \x01(\x0e\x32\".onnx.AttributeProto.AttributeType\x12\t\n\x01\x66\x18\x02 \x01(\x02\x12\t\n\x01i\x18\x03 \x01(\x03\x12\t\n\x01s\x18\x04 \x01(\x0c\x12\x1c\n\x01t\x18\x05 \x01(\x0b\x32\x11.onnx.TensorProto\x12\x1b\n\x01g\x18\x06 \x01(\x0b\x32\x10.onnx.GraphProto\x12.\n\rsparse_tensor\x18\x16 \x01(\x0b\x32\x17.onnx.SparseTensorProto\x12\x1b\n\x02tp\x18\x0e \x01(\x0b\x32\x0f.onnx.TypeProto\x12\x0e\n\x06\x66loats\x18\x07 \x03(\x02\x12\x0c\n\x04ints\x18\x08 \x03(\x03\x12\x0f\n\x07strings\x18\t \x03(\x0c\x12\"\n\x07tensors\x18\n \x03(\x0b\x32\x11.onnx.TensorProto\x12 \n\x06graphs\x18\x0b \x03(\x0b\x32\x10.onnx.GraphProto\x12/\n\x0esparse_tensors\x18\x17 \x03(\x0b\x32\x17.onnx.SparseTensorProto\x12$\n\x0btype_protos\x18\x0f \x03(\x0b\x32\x0f.onnx.TypeProto\"\xd9\x01\n\rAttributeType\x12\r\n\tUNDEFINED\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\x07\n\x03INT\x10\x02\x12\n\n\x06STRING\x10\x03\x12\n\n\x06TENSOR\x10\x04\x12\t\n\x05GRAPH\x10\x05\x12\x11\n\rSPARSE_TENSOR\x10\x0b\x12\x0e\n\nTYPE_PROTO\x10\r\x12\n\n\x06\x46LOATS\x10\x06\x12\x08\n\x04INTS\x10\x07\x12\x0b\n\x07STRINGS\x10\x08\x12\x0b\n\x07TENSORS\x10\t\x12\n\n\x06GRAPHS\x10\n\x12\x12\n\x0eSPARSE_TENSORS\x10\x0c\x12\x0f\n\x0bTYPE_PROTOS\x10\x0eJ\x04\x08\x0c\x10\rJ\x04\x08\x10\x10\x14R\x01v\"\x87\x01\n\x0eValueInfoProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1d\n\x04type\x18\x02 \x01(\x0b\x32\x0f.onnx.TypeProto\x12\x12\n\ndoc_string\x18\x03 \x01(\t\x12\x34\n\x0emetadata_props\x18\x04 \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\"\xa1\x02\n\tNodeProto\x12\r\n\x05input\x18\x01 \x03(\t\x12\x0e\n\x06output\x18\x02 \x03(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x0f\n\x07op_type\x18\x04 \x01(\t\x12\x0e\n\x06\x64omain\x18\x07 \x01(\t\x12\x10\n\x08overload\x18\x08 \x01(\t\x12\'\n\tattribute\x18\x05 \x03(\x0b\x32\x14.onnx.AttributeProto\x12\x12\n\ndoc_string\x18\x06 \x01(\t\x12\x34\n\x0emetadata_props\x18\t \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\x12\x41\n\x15\x64\x65vice_configurations\x18\n \x03(\x0b\x32\".onnx.NodeDeviceConfigurationProto\"2\n\x14IntIntListEntryProto\x12\x0b\n\x03key\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x03(\x03\"\x80\x01\n\x1cNodeDeviceConfigurationProto\x12\x18\n\x10\x63onfiguration_id\x18\x01 \x01(\t\x12.\n\rsharding_spec\x18\x02 \x03(\x0b\x32\x17.onnx.ShardingSpecProto\x12\x16\n\x0epipeline_stage\x18\x03 \x01(\x05\"\xa3\x01\n\x11ShardingSpecProto\x12\x13\n\x0btensor_name\x18\x01 \x01(\t\x12\x0e\n\x06\x64\x65vice\x18\x02 \x03(\x03\x12=\n\x19index_to_device_group_map\x18\x03 \x03(\x0b\x32\x1a.onnx.IntIntListEntryProto\x12*\n\x0bsharded_dim\x18\x04 \x03(\x0b\x32\x15.onnx.ShardedDimProto\"U\n\x0fShardedDimProto\x12\x0c\n\x04\x61xis\x18\x01 \x01(\x03\x12\x34\n\x0fsimple_sharding\x18\x02 \x03(\x0b\x32\x1b.onnx.SimpleShardedDimProto\"\\\n\x15SimpleShardedDimProto\x12\x13\n\tdim_value\x18\x01 \x01(\x03H\x00\x12\x13\n\tdim_param\x18\x02 \x01(\tH\x00\x12\x12\n\nnum_shards\x18\x03 \x01(\x03\x42\x05\n\x03\x64im\"\xd6\x01\n\x11TrainingInfoProto\x12(\n\x0einitialization\x18\x01 \x01(\x0b\x32\x10.onnx.GraphProto\x12#\n\talgorithm\x18\x02 \x01(\x0b\x32\x10.onnx.GraphProto\x12<\n\x16initialization_binding\x18\x03 \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\x12\x34\n\x0eupdate_binding\x18\x04 \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\"\xa2\x03\n\nModelProto\x12\x12\n\nir_version\x18\x01 \x01(\x03\x12.\n\x0copset_import\x18\x08 \x03(\x0b\x32\x18.onnx.OperatorSetIdProto\x12\x15\n\rproducer_name\x18\x02 \x01(\t\x12\x18\n\x10producer_version\x18\x03 \x01(\t\x12\x0e\n\x06\x64omain\x18\x04 \x01(\t\x12\x15\n\rmodel_version\x18\x05 \x01(\x03\x12\x12\n\ndoc_string\x18\x06 \x01(\t\x12\x1f\n\x05graph\x18\x07 \x01(\x0b\x32\x10.onnx.GraphProto\x12\x34\n\x0emetadata_props\x18\x0e \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\x12.\n\rtraining_info\x18\x14 \x03(\x0b\x32\x17.onnx.TrainingInfoProto\x12&\n\tfunctions\x18\x19 \x03(\x0b\x32\x13.onnx.FunctionProto\x12\x35\n\rconfiguration\x18\x1a \x03(\x0b\x32\x1e.onnx.DeviceConfigurationProto\"M\n\x18\x44\x65viceConfigurationProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0bnum_devices\x18\x02 \x01(\x05\x12\x0e\n\x06\x64\x65vice\x18\x03 \x03(\t\"4\n\x16StringStringEntryProto\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"k\n\x10TensorAnnotation\x12\x13\n\x0btensor_name\x18\x01 \x01(\t\x12\x42\n\x1cquant_parameter_tensor_names\x18\x02 \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\"\xd4\x03\n\nGraphProto\x12\x1d\n\x04node\x18\x01 \x03(\x0b\x32\x0f.onnx.NodeProto\x12\x0c\n\x04name\x18\x02 \x01(\t\x12&\n\x0binitializer\x18\x05 \x03(\x0b\x32\x11.onnx.TensorProto\x12\x33\n\x12sparse_initializer\x18\x0f \x03(\x0b\x32\x17.onnx.SparseTensorProto\x12\x12\n\ndoc_string\x18\n \x01(\t\x12#\n\x05input\x18\x0b \x03(\x0b\x32\x14.onnx.ValueInfoProto\x12$\n\x06output\x18\x0c \x03(\x0b\x32\x14.onnx.ValueInfoProto\x12(\n\nvalue_info\x18\r \x03(\x0b\x32\x14.onnx.ValueInfoProto\x12\x37\n\x17quantization_annotation\x18\x0e \x03(\x0b\x32\x16.onnx.TensorAnnotation\x12\x34\n\x0emetadata_props\x18\x10 \x03(\x0b\x32\x1c.onnx.StringStringEntryProtoJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x06\x10\nR\nir_versionR\x10producer_versionR\x0cproducer_tagR\x06\x64omain\"\x82\x07\n\x0bTensorProto\x12\x0c\n\x04\x64ims\x18\x01 \x03(\x03\x12\x11\n\tdata_type\x18\x02 \x01(\x05\x12*\n\x07segment\x18\x03 \x01(\x0b\x32\x19.onnx.TensorProto.Segment\x12\x16\n\nfloat_data\x18\x04 \x03(\x02\x42\x02\x10\x01\x12\x16\n\nint32_data\x18\x05 \x03(\x05\x42\x02\x10\x01\x12\x13\n\x0bstring_data\x18\x06 \x03(\x0c\x12\x16\n\nint64_data\x18\x07 \x03(\x03\x42\x02\x10\x01\x12\x0c\n\x04name\x18\x08 \x01(\t\x12\x12\n\ndoc_string\x18\x0c \x01(\t\x12\x10\n\x08raw_data\x18\t \x01(\x0c\x12\x33\n\rexternal_data\x18\r \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\x12\x35\n\rdata_location\x18\x0e \x01(\x0e\x32\x1e.onnx.TensorProto.DataLocation\x12\x17\n\x0b\x64ouble_data\x18\n \x03(\x01\x42\x02\x10\x01\x12\x17\n\x0buint64_data\x18\x0b \x03(\x04\x42\x02\x10\x01\x12\x34\n\x0emetadata_props\x18\x10 \x03(\x0b\x32\x1c.onnx.StringStringEntryProto\x1a%\n\x07Segment\x12\r\n\x05\x62\x65gin\x18\x01 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x03\"\xee\x02\n\x08\x44\x61taType\x12\r\n\tUNDEFINED\x10\x00\x12\t\n\x05\x46LOAT\x10\x01\x12\t\n\x05UINT8\x10\x02\x12\x08\n\x04INT8\x10\x03\x12\n\n\x06UINT16\x10\x04\x12\t\n\x05INT16\x10\x05\x12\t\n\x05INT32\x10\x06\x12\t\n\x05INT64\x10\x07\x12\n\n\x06STRING\x10\x08\x12\x08\n\x04\x42OOL\x10\t\x12\x0b\n\x07\x46LOAT16\x10\n\x12\n\n\x06\x44OUBLE\x10\x0b\x12\n\n\x06UINT32\x10\x0c\x12\n\n\x06UINT64\x10\r\x12\r\n\tCOMPLEX64\x10\x0e\x12\x0e\n\nCOMPLEX128\x10\x0f\x12\x0c\n\x08\x42\x46LOAT16\x10\x10\x12\x10\n\x0c\x46LOAT8E4M3FN\x10\x11\x12\x12\n\x0e\x46LOAT8E4M3FNUZ\x10\x12\x12\x0e\n\nFLOAT8E5M2\x10\x13\x12\x12\n\x0e\x46LOAT8E5M2FNUZ\x10\x14\x12\t\n\x05UINT4\x10\x15\x12\x08\n\x04INT4\x10\x16\x12\x0e\n\nFLOAT4E2M1\x10\x17\x12\x0e\n\nFLOAT8E8M0\x10\x18\x12\t\n\x05UINT2\x10\x19\x12\x08\n\x04INT2\x10\x1a\")\n\x0c\x44\x61taLocation\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0c\n\x08\x45XTERNAL\x10\x01\"h\n\x11SparseTensorProto\x12!\n\x06values\x18\x01 \x01(\x0b\x32\x11.onnx.TensorProto\x12\"\n\x07indices\x18\x02 \x01(\x0b\x32\x11.onnx.TensorProto\x12\x0c\n\x04\x64ims\x18\x03 \x03(\x03\"\x95\x01\n\x10TensorShapeProto\x12-\n\x03\x64im\x18\x01 \x03(\x0b\x32 .onnx.TensorShapeProto.Dimension\x1aR\n\tDimension\x12\x13\n\tdim_value\x18\x01 \x01(\x03H\x00\x12\x13\n\tdim_param\x18\x02 \x01(\tH\x00\x12\x12\n\ndenotation\x18\x03 \x01(\tB\x07\n\x05value\"\xa5\x05\n\tTypeProto\x12-\n\x0btensor_type\x18\x01 \x01(\x0b\x32\x16.onnx.TypeProto.TensorH\x00\x12\x31\n\rsequence_type\x18\x04 \x01(\x0b\x32\x18.onnx.TypeProto.SequenceH\x00\x12\'\n\x08map_type\x18\x05 \x01(\x0b\x32\x13.onnx.TypeProto.MapH\x00\x12\x31\n\roptional_type\x18\t \x01(\x0b\x32\x18.onnx.TypeProto.OptionalH\x00\x12:\n\x12sparse_tensor_type\x18\x08 \x01(\x0b\x32\x1c.onnx.TypeProto.SparseTensorH\x00\x12-\n\x0bopaque_type\x18\x07 \x01(\x0b\x32\x16.onnx.TypeProto.OpaqueH\x00\x12\x12\n\ndenotation\x18\x06 \x01(\t\x1a\x42\n\x06Tensor\x12\x11\n\telem_type\x18\x01 \x01(\x05\x12%\n\x05shape\x18\x02 \x01(\x0b\x32\x16.onnx.TensorShapeProto\x1a.\n\x08Sequence\x12\"\n\telem_type\x18\x01 \x01(\x0b\x32\x0f.onnx.TypeProto\x1a<\n\x03Map\x12\x10\n\x08key_type\x18\x01 \x01(\x05\x12#\n\nvalue_type\x18\x02 \x01(\x0b\x32\x0f.onnx.TypeProto\x1a.\n\x08Optional\x12\"\n\telem_type\x18\x01 \x01(\x0b\x32\x0f.onnx.TypeProto\x1aH\n\x0cSparseTensor\x12\x11\n\telem_type\x18\x01 \x01(\x05\x12%\n\x05shape\x18\x02 \x01(\x0b\x32\x16.onnx.TensorShapeProto\x1a&\n\x06Opaque\x12\x0e\n\x06\x64omain\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\tB\x07\n\x05value\"5\n\x12OperatorSetIdProto\x12\x0e\n\x06\x64omain\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\"\x86\x03\n\rFunctionProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05input\x18\x04 \x03(\t\x12\x0e\n\x06output\x18\x05 \x03(\t\x12\x11\n\tattribute\x18\x06 \x03(\t\x12-\n\x0f\x61ttribute_proto\x18\x0b \x03(\x0b\x32\x14.onnx.AttributeProto\x12\x1d\n\x04node\x18\x07 \x03(\x0b\x32\x0f.onnx.NodeProto\x12\x12\n\ndoc_string\x18\x08 \x01(\t\x12.\n\x0copset_import\x18\t \x03(\x0b\x32\x18.onnx.OperatorSetIdProto\x12\x0e\n\x06\x64omain\x18\n \x01(\t\x12\x10\n\x08overload\x18\r \x01(\t\x12(\n\nvalue_info\x18\x0c \x03(\x0b\x32\x14.onnx.ValueInfoProto\x12\x34\n\x0emetadata_props\x18\x0e \x03(\x0b\x32\x1c.onnx.StringStringEntryProtoJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04R\rsince_versionR\x06status*\xe7\x02\n\x07Version\x12\x12\n\x0e_START_VERSION\x10\x00\x12\x19\n\x15IR_VERSION_2017_10_10\x10\x01\x12\x19\n\x15IR_VERSION_2017_10_30\x10\x02\x12\x18\n\x14IR_VERSION_2017_11_3\x10\x03\x12\x18\n\x14IR_VERSION_2019_1_22\x10\x04\x12\x18\n\x14IR_VERSION_2019_3_18\x10\x05\x12\x18\n\x14IR_VERSION_2019_9_19\x10\x06\x12\x17\n\x13IR_VERSION_2020_5_8\x10\x07\x12\x18\n\x14IR_VERSION_2021_7_30\x10\x08\x12\x17\n\x13IR_VERSION_2023_5_5\x10\t\x12\x18\n\x14IR_VERSION_2024_3_25\x10\n\x12\x19\n\x15IR_VERSION_2025_05_12\x10\x0b\x12\x19\n\x15IR_VERSION_2025_08_26\x10\x0c\x12\x0e\n\nIR_VERSION\x10\r*.\n\x0eOperatorStatus\x12\x10\n\x0c\x45XPERIMENTAL\x10\x00\x12\n\n\x06STABLE\x10\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'onnx.onnx_ml_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_TENSORPROTO'].fields_by_name['float_data']._loaded_options = None
  _globals['_TENSORPROTO'].fields_by_name['float_data']._serialized_options = b'\020\001'
  _globals['_TENSORPROTO'].fields_by_name['int32_data']._loaded_options = None
  _globals['_TENSORPROTO'].fields_by_name['int32_data']._serialized_options = b'\020\001'
  _globals['_TENSORPROTO'].fields_by_name['int64_data']._loaded_options = None
  _globals['_TENSORPROTO'].fields_by_name['int64_data']._serialized_options = b'\020\001'
  _globals['_TENSORPROTO'].fields_by_name['double_data']._loaded_options = None
  _globals['_TENSORPROTO'].fields_by_name['double_data']._serialized_options = b'\020\001'
  _globals['_TENSORPROTO'].fields_by_name['uint64_data']._loaded_options = None
  _globals['_TENSORPROTO'].fields_by_name['uint64_data']._serialized_options = b'\020\001'
  _globals['_VERSION']._serialized_start=5361
  _globals['_VERSION']._serialized_end=5720
  _globals['_OPERATORSTATUS']._serialized_start=5722
  _globals['_OPERATORSTATUS']._serialized_end=5768
  _globals['_ATTRIBUTEPROTO']._serialized_start=29
  _globals['_ATTRIBUTEPROTO']._serialized_end=760
  _globals['_ATTRIBUTEPROTO_ATTRIBUTETYPE']._serialized_start=528
  _globals['_ATTRIBUTEPROTO_ATTRIBUTETYPE']._serialized_end=745
  _globals['_VALUEINFOPROTO']._serialized_start=763
  _globals['_VALUEINFOPROTO']._serialized_end=898
  _globals['_NODEPROTO']._serialized_start=901
  _globals['_NODEPROTO']._serialized_end=1190
  _globals['_INTINTLISTENTRYPROTO']._serialized_start=1192
  _globals['_INTINTLISTENTRYPROTO']._serialized_end=1242
  _globals['_NODEDEVICECONFIGURATIONPROTO']._serialized_start=1245
  _globals['_NODEDEVICECONFIGURATIONPROTO']._serialized_end=1373
  _globals['_SHARDINGSPECPROTO']._serialized_start=1376
  _globals['_SHARDINGSPECPROTO']._serialized_end=1539
  _globals['_SHARDEDDIMPROTO']._serialized_start=1541
  _globals['_SHARDEDDIMPROTO']._serialized_end=1626
  _globals['_SIMPLESHARDEDDIMPROTO']._serialized_start=1628
  _globals['_SIMPLESHARDEDDIMPROTO']._serialized_end=1720
  _globals['_TRAININGINFOPROTO']._serialized_start=1723
  _globals['_TRAININGINFOPROTO']._serialized_end=1937
  _globals['_MODELPROTO']._serialized_start=1940
  _globals['_MODELPROTO']._serialized_end=2358
  _globals['_DEVICECONFIGURATIONPROTO']._serialized_start=2360
  _globals['_DEVICECONFIGURATIONPROTO']._serialized_end=2437
  _globals['_STRINGSTRINGENTRYPROTO']._serialized_start=2439
  _globals['_STRINGSTRINGENTRYPROTO']._serialized_end=2491
  _globals['_TENSORANNOTATION']._serialized_start=2493
  _globals['_TENSORANNOTATION']._serialized_end=2600
  _globals['_GRAPHPROTO']._serialized_start=2603
  _globals['_GRAPHPROTO']._serialized_end=3071
  _globals['_TENSORPROTO']._serialized_start=3074
  _globals['_TENSORPROTO']._serialized_end=3972
  _globals['_TENSORPROTO_SEGMENT']._serialized_start=3523
  _globals['_TENSORPROTO_SEGMENT']._serialized_end=3560
  _globals['_TENSORPROTO_DATATYPE']._serialized_start=3563
  _globals['_TENSORPROTO_DATATYPE']._serialized_end=3929
  _globals['_TENSORPROTO_DATALOCATION']._serialized_start=3931
  _globals['_TENSORPROTO_DATALOCATION']._serialized_end=3972
  _globals['_SPARSETENSORPROTO']._serialized_start=3974
  _globals['_SPARSETENSORPROTO']._serialized_end=4078
  _globals['_TENSORSHAPEPROTO']._serialized_start=4081
  _globals['_TENSORSHAPEPROTO']._serialized_end=4230
  _globals['_TENSORSHAPEPROTO_DIMENSION']._serialized_start=4148
  _globals['_TENSORSHAPEPROTO_DIMENSION']._serialized_end=4230
  _globals['_TYPEPROTO']._serialized_start=4233
  _globals['_TYPEPROTO']._serialized_end=4910
  _globals['_TYPEPROTO_TENSOR']._serialized_start=4563
  _globals['_TYPEPROTO_TENSOR']._serialized_end=4629
  _globals['_TYPEPROTO_SEQUENCE']._serialized_start=4631
  _globals['_TYPEPROTO_SEQUENCE']._serialized_end=4677
  _globals['_TYPEPROTO_MAP']._serialized_start=4679
  _globals['_TYPEPROTO_MAP']._serialized_end=4739
  _globals['_TYPEPROTO_OPTIONAL']._serialized_start=4741
  _globals['_TYPEPROTO_OPTIONAL']._serialized_end=4787
  _globals['_TYPEPROTO_SPARSETENSOR']._serialized_start=4789
  _globals['_TYPEPROTO_SPARSETENSOR']._serialized_end=4861
  _globals['_TYPEPROTO_OPAQUE']._serialized_start=4863
  _globals['_TYPEPROTO_OPAQUE']._serialized_end=4901
  _globals['_OPERATORSETIDPROTO']._serialized_start=4912
  _globals['_OPERATORSETIDPROTO']._serialized_end=4965
  _globals['_FUNCTIONPROTO']._serialized_start=4968
  _globals['_FUNCTIONPROTO']._serialized_end=5358
# @@protoc_insertion_point(module_scope)



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.op_run import OpRun


def _one_hot(indices, depth, axis=-1, dtype=np.float32):
    values = np.asarray(indices)
    rank = len(values.shape)
    depth_range = np.arange(depth)
    if axis < 0:
        axis += rank + 1
    ls = values.shape[0:axis]
    rs = values.shape[axis:rank]
    new_shape = (1,) * len(ls) + depth_range.shape + (1,) * len(rs)
    targets = np.reshape(depth_range, new_shape)
    values = np.reshape(np.mod(values, depth), (*ls, 1, *rs))
    return np.asarray(targets == values, dtype=dtype)


class OneHot(OpRun):
    def _run(self, indices, depth, values, axis=None):
        off_value, on_value = values
        y = _one_hot(indices, depth, axis=axis, dtype=values.dtype)
        y = y * (on_value - off_value) + off_value
        return (y,)



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

from typing import NamedTuple

import ml_dtypes
import numpy as np

from onnx.onnx_pb import TensorProto


class TensorDtypeMap(NamedTuple):
    np_dtype: np.dtype
    storage_dtype: int
    name: str


# tensor_dtype: (numpy type, storage type, string name)
# The storage type is the type used to store the tensor in the *_data field of
# a TensorProto. All available fields are float_data, int32_data, int64_data,
# string_data, uint64_data and double_data.
TENSOR_TYPE_MAP: dict[int, TensorDtypeMap] = {
    int(TensorProto.FLOAT): TensorDtypeMap(
        np.dtype("float32"), int(TensorProto.FLOAT), "TensorProto.FLOAT"
    ),
    int(TensorProto.UINT8): TensorDtypeMap(
        np.dtype("uint8"), int(TensorProto.INT32), "TensorProto.UINT8"
    ),
    int(TensorProto.INT8): TensorDtypeMap(
        np.dtype("int8"), int(TensorProto.INT32), "TensorProto.INT8"
    ),
    int(TensorProto.UINT16): TensorDtypeMap(
        np.dtype("uint16"), int(TensorProto.INT32), "TensorProto.UINT16"
    ),
    int(TensorProto.INT16): TensorDtypeMap(
        np.dtype("int16"), int(TensorProto.INT32), "TensorProto.INT16"
    ),
    int(TensorProto.INT32): TensorDtypeMap(
        np.dtype("int32"), int(TensorProto.INT32), "TensorProto.INT32"
    ),
    int(TensorProto.INT64): TensorDtypeMap(
        np.dtype("int64"), int(TensorProto.INT64), "TensorProto.INT64"
    ),
    int(TensorProto.BOOL): TensorDtypeMap(
        np.dtype("bool"), int(TensorProto.INT32), "TensorProto.BOOL"
    ),
    int(TensorProto.FLOAT16): TensorDtypeMap(
        np.dtype("float16"), int(TensorProto.INT32), "TensorProto.FLOAT16"
    ),
    int(TensorProto.BFLOAT16): TensorDtypeMap(
        np.dtype(ml_dtypes.bfloat16),
        int(TensorProto.INT32),
        "TensorProto.BFLOAT16",
    ),
    int(TensorProto.DOUBLE): TensorDtypeMap(
        np.dtype("float64"), int(TensorProto.DOUBLE), "TensorProto.DOUBLE"
    ),
    int(TensorProto.COMPLEX64): TensorDtypeMap(
        np.dtype("complex64"), int(TensorProto.FLOAT), "TensorProto.COMPLEX64"
    ),
    int(TensorProto.COMPLEX128): TensorDtypeMap(
        np.dtype("complex128"),
        int(TensorProto.DOUBLE),
        "TensorProto.COMPLEX128",
    ),
    int(TensorProto.UINT32): TensorDtypeMap(
        np.dtype("uint32"), int(TensorProto.UINT64), "TensorProto.UINT32"
    ),
    int(TensorProto.UINT64): TensorDtypeMap(
        np.dtype("uint64"), int(TensorProto.UINT64), "TensorProto.UINT64"
    ),
    int(TensorProto.STRING): TensorDtypeMap(
        np.dtype("object"), int(TensorProto.STRING), "TensorProto.STRING"
    ),
    int(TensorProto.FLOAT8E4M3FN): TensorDtypeMap(
        np.dtype(ml_dtypes.float8_e4m3fn),
        int(TensorProto.INT32),
        "TensorProto.FLOAT8E4M3FN",
    ),
    int(TensorProto.FLOAT8E4M3FNUZ): TensorDtypeMap(
        np.dtype(ml_dtypes.float8_e4m3fnuz),
        int(TensorProto.INT32),
        "TensorProto.FLOAT8E4M3FNUZ",
    ),
    int(TensorProto.FLOAT8E5M2): TensorDtypeMap(
        np.dtype(ml_dtypes.float8_e5m2),
        int(TensorProto.INT32),
        "TensorProto.FLOAT8E5M2",
    ),
    int(TensorProto.FLOAT8E5M2FNUZ): TensorDtypeMap(
        np.dtype(ml_dtypes.float8_e5m2fnuz),
        int(TensorProto.INT32),
        "TensorProto.FLOAT8E5M2FNUZ",
    ),
    int(TensorProto.UINT4): TensorDtypeMap(
        np.dtype(ml_dtypes.uint4), int(TensorProto.INT32), "TensorProto.UINT4"
    ),
    int(TensorProto.INT4): TensorDtypeMap(
        np.dtype(ml_dtypes.int4), int(TensorProto.INT32), "TensorProto.INT4"
    ),
    int(TensorProto.FLOAT4E2M1): TensorDtypeMap(
        np.dtype(ml_dtypes.float4_e2m1fn),
        int(TensorProto.INT32),
        "TensorProto.FLOAT4E2M1",
    ),
    int(TensorProto.FLOAT8E8M0): TensorDtypeMap(
        np.dtype(ml_dtypes.float8_e8m0fnu),
        int(TensorProto.INT32),
        "TensorProto.FLOAT8E8M0",
    ),
    int(TensorProto.UINT2): TensorDtypeMap(
        np.dtype(ml_dtypes.uint2), int(TensorProto.INT32), "TensorProto.UINT2"
    ),
    int(TensorProto.INT2): TensorDtypeMap(
        np.dtype(ml_dtypes.int2), int(TensorProto.INT32), "TensorProto.INT2"
    ),
}



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


class Equal(Base):
    @staticmethod
    def export() -> None:
        node = onnx.helper.make_node(
            "Equal",
            inputs=["x", "y"],
            outputs=["z"],
        )

        x = (np.random.randn(3, 4, 5) * 10).astype(np.int32)
        y = (np.random.randn(3, 4, 5) * 10).astype(np.int32)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal")

        x = (np.random.randn(3, 4, 5) * 10).astype(np.int8)
        y = (np.random.randn(3, 4, 5) * 10).astype(np.int8)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_int8")

        x = (np.random.randn(3, 4, 5) * 10).astype(np.int16)
        y = (np.random.randn(3, 4, 5) * 10).astype(np.int16)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_int16")

        x = np.random.randint(24, size=(3, 4, 5), dtype=np.uint8)
        y = np.random.randint(24, size=(3, 4, 5), dtype=np.uint8)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_uint8")

        x = np.random.randint(24, size=(3, 4, 5), dtype=np.uint16)
        y = np.random.randint(24, size=(3, 4, 5), dtype=np.uint16)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_uint16")

        x = np.random.randint(24, size=(3, 4, 5), dtype=np.uint32)
        y = np.random.randint(24, size=(3, 4, 5), dtype=np.uint32)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_uint32")

        x = np.random.randint(24, size=(3, 4, 5), dtype=np.uint64)
        y = np.random.randint(24, size=(3, 4, 5), dtype=np.uint64)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_uint64")

    @staticmethod
    def export_equal_broadcast() -> None:
        node = onnx.helper.make_node(
            "Equal",
            inputs=["x", "y"],
            outputs=["z"],
        )

        x = (np.random.randn(3, 4, 5) * 10).astype(np.int32)
        y = (np.random.randn(5) * 10).astype(np.int32)
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_bcast")

    @staticmethod
    def export_equal_string() -> None:
        node = onnx.helper.make_node(
            "Equal",
            inputs=["x", "y"],
            outputs=["z"],
        )
        x = np.array(["string1", "string2"], dtype=np.dtype(object))
        y = np.array(["string1", "string3"], dtype=np.dtype(object))
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_string")

    @staticmethod
    def export_equal_string_broadcast() -> None:
        node = onnx.helper.make_node(
            "Equal",
            inputs=["x", "y"],
            outputs=["z"],
        )
        x = np.array(["string1", "string2"], dtype=np.dtype(object))
        y = np.array(["string1"], dtype=np.dtype(object))
        z = np.equal(x, y)
        expect(node, inputs=[x, y], outputs=[z], name="test_equal_string_broadcast")



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.op_run import OpRun


def _fft(x: np.ndarray, fft_length: int, axis: int) -> np.ndarray:
    """Compute the FFT return the real representation of the complex result."""
    transformed = np.fft.fft(x, n=fft_length, axis=axis)
    real_frequencies = np.real(transformed)
    imaginary_frequencies = np.imag(transformed)
    return np.concatenate(
        (real_frequencies[..., np.newaxis], imaginary_frequencies[..., np.newaxis]),
        axis=-1,
    )


def _cfft(
    x: np.ndarray,
    fft_length: int,
    axis: int,
    onesided: bool,
    normalize: bool,
) -> np.ndarray:
    if x.shape[-1] == 1:
        # The input contains only the real part
        signal = x
    else:
        # The input is a real representation of a complex signal
        slices = [slice(0, x) for x in x.shape]
        slices[-1] = slice(0, x.shape[-1], 2)
        real = x[tuple(slices)]
        slices[-1] = slice(1, x.shape[-1], 2)
        imag = x[tuple(slices)]
        signal = real + 1j * imag

    complex_signals = np.squeeze(signal, -1)
    result = _fft(complex_signals, fft_length, axis=axis)
    # Post process the result based on arguments
    if onesided:
        slices = [slice(0, a) for a in result.shape]
        slices[axis] = slice(0, result.shape[axis] // 2 + 1)
        result = result[tuple(slices)]
    if normalize:
        result /= fft_length
    return result


def _ifft(x: np.ndarray, fft_length: int, axis: int) -> np.ndarray:
    """Standard IFFT: complex input -> complex output."""
    signals = np.fft.ifft(x, n=fft_length, axis=axis)
    real_signals = np.real(signals)
    imaginary_signals = np.imag(signals)
    return np.concatenate(
        (real_signals[..., np.newaxis], imaginary_signals[..., np.newaxis]),
        axis=-1,
    )


def _irfft(x: np.ndarray, fft_length: int, axis: int) -> np.ndarray:
    """IRFFT: one-sided complex input -> full real output."""
    signals = np.fft.irfft(x, n=fft_length, axis=axis)
    # Return real-valued output with last dimension = 1
    return signals[..., np.newaxis]


def _cifft(
    x: np.ndarray, fft_length: int, axis: int, onesided: bool = False
) -> np.ndarray:
    """Complex IFFT wrapper that handles both standard IFFT and IRFFT."""
    # Extract complex values from input
    if x.shape[-1] == 1:
        # Real input (shouldn't happen for IFFT, but handle it)
        frequencies = np.squeeze(x, -1)
    else:
        # Complex input: interleaved real/imaginary
        slices = [slice(0, dim) for dim in x.shape]
        slices[-1] = slice(0, x.shape[-1], 2)
        real = x[tuple(slices)]
        slices[-1] = slice(1, x.shape[-1], 2)
        imag = x[tuple(slices)]
        frequencies = np.squeeze(real, -1) + 1j * np.squeeze(imag, -1)

    if onesided:
        # IRFFT: one-sided complex input -> full real output
        return _irfft(frequencies, fft_length, axis=axis)
    # Standard IFFT: full complex input -> full complex output
    return _ifft(frequencies, fft_length, axis=axis)


class DFT_17(OpRun):
    def _run(
        self,
        x: np.ndarray,
        dft_length: int | None = None,
        axis: int = 1,
        inverse: bool = False,
        onesided: bool = False,
    ) -> tuple[np.ndarray]:
        # Convert to positive axis
        axis = axis % len(x.shape)

        # Set default dft_length based on operation type
        if dft_length is None:
            if inverse and onesided:
                # IRFFT: input is one-sided, default to even length
                dft_length = 2 * (x.shape[axis] - 1)
            else:
                dft_length = x.shape[axis]

        if inverse:
            result = _cifft(x, dft_length, axis=axis, onesided=onesided)
        else:
            result = _cfft(x, dft_length, axis=axis, onesided=onesided, normalize=False)
        return (result.astype(x.dtype),)


class DFT_20(OpRun):
    def _run(
        self,
        x: np.ndarray,
        dft_length: int | None = None,
        axis: int = -2,
        inverse: bool = False,
        onesided: bool = False,
    ) -> tuple[np.ndarray]:
        # Convert to positive axis
        axis = axis % len(x.shape)

        # Set default dft_length based on operation type
        if dft_length is None:
            if inverse and onesided:
                # IRFFT: input is one-sided, default to even length
                dft_length = 2 * (x.shape[axis] - 1)
            else:
                dft_length = x.shape[axis]

        if inverse:
            result = _cifft(x, dft_length, axis=axis, onesided=onesided)
        else:
            result = _cfft(x, dft_length, axis=axis, onesided=onesided, normalize=False)
        return (result.astype(x.dtype),)



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


class StringSplit(Base):
    @staticmethod
    def export_basic() -> None:
        node = onnx.helper.make_node(
            "StringSplit",
            inputs=["x"],
            outputs=["substrings", "length"],
            delimiter=".",
            maxsplit=None,
        )

        x = np.array(["abc.com", "def.net"]).astype(object)

        substrings = np.array([["abc", "com"], ["def", "net"]]).astype(object)

        length = np.array([2, 2], dtype=np.int64)

        expect(
            node,
            inputs=[x],
            outputs=[substrings, length],
            name="test_string_split_basic",
        )

    @staticmethod
    def export_maxsplit() -> None:
        node = onnx.helper.make_node(
            "StringSplit",
            inputs=["x"],
            outputs=["substrings", "length"],
            maxsplit=2,
        )

        x = np.array(
            [["hello world", "def.net"], ["o n n x", "the quick brown fox"]]
        ).astype(object)

        substrings = np.array(
            [
                [["hello", "world", ""], ["def.net", "", ""]],
                [["o", "n", "n x"], ["the", "quick", "brown fox"]],
            ]
        ).astype(object)

        length = np.array([[2, 1], [3, 3]], np.int64)

        expect(
            node,
            inputs=[x],
            outputs=[substrings, length],
            name="test_string_split_maxsplit",
        )

    @staticmethod
    def export_consecutive_delimiters() -> None:
        node = onnx.helper.make_node(
            "StringSplit",
            inputs=["x"],
            outputs=["substrings", "length"],
            delimiter="-",
            maxsplit=None,
        )

        x = np.array(["o-n-n--x-", "o-n----nx"]).astype(object)

        substrings = np.array(
            [["o", "n", "n", "", "x", ""], ["o", "n", "", "", "", "nx"]]
        ).astype(object)

        length = np.array([6, 6], dtype=np.int64)

        expect(
            node,
            inputs=[x],
            outputs=[substrings, length],
            name="test_string_split_consecutive_delimiters",
        )

    @staticmethod
    def export_empty_string_delimiter() -> None:
        for delimiter, test_name in (
            ("", "test_string_split_empty_string_delimiter"),
            (None, "test_string_split_no_delimiter"),
        ):
            node = onnx.helper.make_node(
                "StringSplit",
                inputs=["x"],
                outputs=["substrings", "length"],
                delimiter=delimiter,
                maxsplit=None,
            )

            x = np.array(
                ["hello world !", "  hello   world !", " hello world   ! "]
            ).astype(object)

            substrings = np.array(
                [
                    ["hello", "world", "!"],
                    ["hello", "world", "!"],
                    ["hello", "world", "!"],
                ]
            ).astype(object)

            length = np.array([3, 3, 3], dtype=np.int64)

            expect(
                node,
                inputs=[x],
                outputs=[substrings, length],
                name=test_name,
            )

    @staticmethod
    def export_empty_string_split() -> None:
        node = onnx.helper.make_node(
            "StringSplit",
            inputs=["x"],
            outputs=["substrings", "length"],
            delimiter=None,
            maxsplit=None,
        )

        x = np.array([]).astype(object)

        substrings = np.array([]).astype(object).reshape(0, 0)

        length = np.array([], dtype=np.int64)

        expect(
            node,
            inputs=[x],
            outputs=[substrings, length],
            name="test_string_split_empty_tensor",
            output_type_protos=[
                onnx.helper.make_tensor_type_proto(onnx.TensorProto.STRING, (0, None)),
                None,
            ],
        )



# Copyright (c) ONNX Project Contributors

# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

from onnx.reference.ops._op import OpRunUnaryNum


class HardSigmoid(OpRunUnaryNum):
    def _run(self, x, alpha=None, beta=None):
        alpha = alpha or self.alpha
        beta = beta or self.beta
        y = np.maximum(0, np.minimum(1, x * alpha + beta)).astype(x.dtype)
        return (y,)



# Copyright (c) ONNX Project Contributors
#
# SPDX-License-Identifier: Apache-2.0
from __future__ import annotations

import numpy as np

import onnx
from onnx.backend.test.case.base import Base
from onnx.backend.test.case.node import expect


def hardswish(x: np.ndarray) -> np.ndarray:
    alfa = float(1 / 6)
    beta = 0.5
    return x * np.maximum(0, np.minimum(1, alfa * x + beta))


class HardSwish(Base):
    @staticmethod
    def export() -> None:
        node = onnx.helper.make_node(
            "HardSwish",
            inputs=["x"],
            outputs=["y"],
        )
        x = np.random.randn(3, 4, 5).astype(np.float32)
        y = hardswish(x)

        expect(node, inputs=[x], outputs=[y], name="test_hardswish")
